@@ -1,11 +1,13 @@
 #include "../../src/anjuta.h"
 #include "../../src/anjuta_info.h"
+#include "../../src/anjuta-plugins.h"
 
 gchar   *GetDescr       (void);
 glong    GetVersion     (void);
 gboolean Init           (GModule *self, void **pUserData, AnjutaApp* p);
 void     CleanUp        (GModule *self, void *pUserData, AnjutaApp* p);
 void     Activate       (GModule *self, void *pUserData, AnjutaApp* p);
+gchar * GetMenu         (void);
 gchar   *GetMenuTitle   (GModule *self, void *pUserData);
 gchar   *GetTooltipText (GModule *self, void *pUserData);
 
@@ -22,6 +24,13 @@ GetVersion()
 	return 0x10000L ;
 }
 
+/* Place this entry under the 'View' menu */
+gchar *
+GetMenu()
+{
+	return g_strdup("view");
+}
+
 gboolean
 Init( GModule *self, void **pUserData, AnjutaApp* p )
 {
