@@ -1,6 +1,6 @@
 /*
     about.h
-    Copyright (C) 2000  Kh. Naba Kumar Singh
+    Copyright (C) 2002 Naba Kumar
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,9 +16,5 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
-#ifndef _ABOUT_H_
-#define _ABOUT_H_
 
-GtkWidget* create_about_gui (void);
-
-#endif
+GtkWidget *about_box_new (void);
