@@ -12,6 +12,7 @@ extern "C"
 
 typedef struct _SymbolFileInfo
 {
+	char *sym_name;
 	struct
 	{
 		char *name;
@@ -28,12 +29,22 @@ typedef struct _AnSymbolView
 {
 	GtkWidget *win;
 	GtkWidget *tree;
-	GtkWidget *menu;
+	struct
+	{
+		GtkWidget *top;
+		GtkWidget *goto_decl;
+		GtkWidget *goto_def;
+		GtkWidget *find;
+		GtkWidget *refresh;
+		GtkWidget *docked;
+	} menu;
 	SymbolFileInfo *sinfo;
 } AnSymbolView;
 
-AnSymbolView *sv_populate(void);
-void sv_clear(void);
+AnSymbolView *sv_populate (gboolean full);
+void	      sv_clear (void);
+GList*        sv_get_node_expansion_states (void);
+void          sv_set_node_expansion_states (GList *expansion_states);
 
 #ifdef __cplusplus
 }
