@@ -1,6 +1,6 @@
 /*
     anjuta_cbs.c
-    Copyright (C) 2000  Kh. Naba Kumar Singh
+    Copyright (C) 2000  Naba Kumar <naba@gnome.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -33,7 +33,6 @@
 #include "fileselection.h"
 #include "utilities.h"
 #include "resources.h"
-#include "messagebox.h"
 #include "launcher.h"
 #include "debugger.h"
 
@@ -49,14 +48,19 @@ on_anjuta_session_die(GnomeClient * client, gpointer data)
 
 /* Saves the current anjuta session */
 gint on_anjuta_session_save_yourself (GnomeClient * client, gint phase,
-		       GnomeSaveStyle s_style, gint shutdown,
-		       GnomeInteractStyle i_style, gint fast, gpointer data)
+									  GnomeSaveStyle s_style, gint shutdown,
+									  GnomeInteractStyle i_style, gint fast,
+									  gpointer data)
 {
 	gchar *argv[] = { "rm",	"-rf", NULL};
-	gchar *prefix, **res_argv;
+	const gchar *prefix;
+	gchar **res_argv;
 	gint res_argc, counter;
 	GList* node;
 
+#ifdef DEBUG
+	g_message ("Going to save session ...");
+#endif
 	prefix = gnome_client_get_config_prefix (client);
 	argv[2] = gnome_config_get_real_path (prefix);
 	gnome_client_set_discard_command (client, 3, argv);
@@ -94,21 +98,30 @@ gint on_anjuta_session_save_yourself (GnomeClient * client, gint phase,
 gint on_anjuta_delete (GtkWidget * w, GdkEvent * event, gpointer data)
 {
 	TextEditor *te;
+	ProjectDBase *p;
 	GList *list;
 	gboolean file_not_saved;
 	gint max_recent_files, max_recent_prjs;
 	
 	if (!app) return TRUE;
 	file_not_saved = FALSE;
-	max_recent_files = preferences_get_int (app->preferences, MAXIMUM_RECENT_FILES);
-	max_recent_prjs = preferences_get_int (app->preferences, MAXIMUM_RECENT_PROJECTS);
+	p = app->project_dbase;
+	
+	max_recent_files = 
+		anjuta_preferences_get_int (ANJUTA_PREFERENCES (app->preferences),
+									MAXIMUM_RECENT_FILES);
+	max_recent_prjs =
+		anjuta_preferences_get_int (ANJUTA_PREFERENCES (app->preferences),
+									MAXIMUM_RECENT_PROJECTS);
 	list = app->text_editor_list;
 	while (list)
 	{
 		te = list->data;
 		if (te->full_filename)
-			app->recent_files = update_string_list (app->recent_files,
-					te->full_filename, max_recent_files);
+			app->recent_files =
+				glist_path_dedup(update_string_list (app->recent_files,
+													 te->full_filename,
+													 max_recent_files));
 		if (!text_editor_is_saved (te))
 			file_not_saved = TRUE;
 		list = g_list_next (list);
@@ -116,61 +129,278 @@ gint on_anjuta_delete (GtkWidget * w, GdkEvent * event, gpointer data)
 	if (app->project_dbase->project_is_open)
 	{
 		app->recent_projects =
-			update_string_list (app->recent_projects,
-					    app->project_dbase->proj_filename,
-					    max_recent_prjs);
+				glist_path_dedup(update_string_list (app->recent_projects,
+												app->project_dbase->proj_filename,
+													 max_recent_files));
 	}
-	anjuta_save_settings ();
-
-	if (file_not_saved || (!app->project_dbase->is_saved && app->project_dbase->project_is_open))
+	// Check for project save status.
+	if (p->project_is_open)
 	{
-		messagebox2 (GNOME_MESSAGE_BOX_QUESTION,
-			     _("Some files (or the project) are not saved.\n"
-				"Do you still want to exit?"),
-			     GNOME_STOCK_BUTTON_YES,
-			     GNOME_STOCK_BUTTON_NO,
-			     GTK_SIGNAL_FUNC
-			     (on_anjuta_exit_yes_clicked), NULL,
-			     NULL);
+		if (p->is_saved == FALSE)
+		{
+			GtkWidget *dialog;
+			gint but;
+			
+			dialog = create_project_confirm_dlg (p->widgets.window);
+			but = gtk_dialog_run (GTK_DIALOG (dialog));
+			switch (but)
+			{
+				case GTK_RESPONSE_YES:
+					project_dbase_save_project (p);
+					break;
+				case GTK_RESPONSE_NO:
+					p->is_saved = TRUE;
+					break;
+				default:
+					gtk_widget_destroy (dialog);
+					return TRUE;
+			}
+			gtk_widget_destroy (dialog);
+		}
+		/* Save session.... */
+		project_dbase_save_session(p);
+	}
+
+	// Check for file save status.	
+	if (file_not_saved)
+	{
+		GtkWidget *dialog;
+		gint response;
+		dialog = gtk_message_dialog_new (GTK_WINDOW (app->widgets.window),
+										 GTK_DIALOG_DESTROY_WITH_PARENT,
+										 GTK_MESSAGE_QUESTION,
+										 GTK_BUTTONS_NONE,
+										 _("One or more files are not saved.\n"
+										 "Do you still want to exit?"));
+		gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+								GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+								GTK_STOCK_QUIT,   GTK_RESPONSE_YES,
+								NULL);
+		response = gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+		if (response == GTK_RESPONSE_YES)
+			anjuta_clean_exit ();
 		return TRUE;
 	}
 	else
-	{
-		on_anjuta_exit_yes_clicked (NULL, NULL);
-	}
+		anjuta_clean_exit ();
 	return TRUE;
 }
 
-void
-on_anjuta_exit_yes_clicked (GtkButton * b, gpointer data)
-{
-	anjuta_clean_exit ();
-}
-
 void
 on_anjuta_destroy (GtkWidget * w, gpointer data)
 {
 	/* Nothing to be done here */
 }
 
+/*! state flag for Ctrl-TAB */
+static gboolean g_tabbing = FALSE;
+
 void
 on_anjuta_notebook_switch_page (GtkNotebook * notebook,
-				GtkNotebookPage * page,
-				gint page_num, gpointer user_data)
+								GtkNotebookPage * page,
+								gint page_num, gpointer user_data)
 {
+	GtkWidget *widget;
+	
 	anjuta_set_current_text_editor (anjuta_get_notebook_text_editor
-					(page_num));
+									(page_num));
+	if (!g_tabbing &&
+		!anjuta_preferences_get_int (app->preferences, EDITOR_TABS_ORDERING) && 
+		anjuta_preferences_get_int (app->preferences, EDITOR_TABS_RECENT_FIRST))
+	{
+		/* TTimo: reorder so that the most recently used files are always
+		 * at the beginning of the tab list
+		 */
+		widget = gtk_notebook_get_nth_page (notebook, page_num);
+		gtk_notebook_reorder_child (notebook, widget, 0);
+	}
 	anjuta_grab_text_focus ();
 }
 
 void
-on_anjuta_dnd_drop (gchar* filename, gpointer data)
+on_anjuta_dnd_drop (const gchar* filename, gpointer data)
 {
 	anjuta_goto_file_line (filename, 0);
 }
 
+gboolean
+on_anjuta_window_focus_in_event (GtkWidget * widget,
+				 GdkEventFocus * event, gpointer user_data)
+{
+	gint page_num;
+	TextEditor *te;
+
+	if (g_list_length (GTK_NOTEBOOK (app->widgets.notebook)->children) >
+	    0)
+	{
+		page_num =
+			gtk_notebook_get_current_page (GTK_NOTEBOOK
+						       (app->widgets.
+							notebook));
+		if (page_num >= 0)
+			te = anjuta_get_notebook_text_editor (page_num);
+		else
+			te = NULL;
+	}
+	else if (app->text_editor_list == NULL)
+	{
+		te = NULL;
+	}
+	else
+	{
+		/* If there is TextEditor in the list, but no page 
+		 * * in the notebook, we don't change the current
+		 * * TextEditor
+		 */
+		return FALSE;
+	}
+	if (te == anjuta_get_current_text_editor ())
+		return FALSE;
+	anjuta_set_current_text_editor (te);
+	return FALSE;
+}
+
+enum {
+	m___ = 0,
+	mS__ = GDK_SHIFT_MASK,
+	m_C_ = GDK_CONTROL_MASK,
+	m__M = GDK_MOD1_MASK,
+	mSC_ = GDK_SHIFT_MASK | GDK_CONTROL_MASK,
+};
+
+enum {
+	ID_NEXTBUFFER = 1, /* Note: the value mustn't be 0 ! */
+	ID_PREVBUFFER,
+	ID_FIRSTBUFFER
+};
+
+typedef struct {
+	int modifiers;
+	unsigned int gdk_key;
+	int id;
+} ShortcutMapping;
+
+static ShortcutMapping global_keymap[] = {
+	{ m_C_, GDK_Tab,		 ID_NEXTBUFFER },
+	{ mSC_, GDK_ISO_Left_Tab, ID_PREVBUFFER },
+	{ m__M, GDK_1, ID_FIRSTBUFFER },
+	{ m__M, GDK_2, ID_FIRSTBUFFER + 1},
+	{ m__M, GDK_3, ID_FIRSTBUFFER + 2},
+	{ m__M, GDK_4, ID_FIRSTBUFFER + 3},
+	{ m__M, GDK_5, ID_FIRSTBUFFER + 4},
+	{ m__M, GDK_6, ID_FIRSTBUFFER + 5},
+	{ m__M, GDK_7, ID_FIRSTBUFFER + 6},
+	{ m__M, GDK_8, ID_FIRSTBUFFER + 7},
+	{ m__M, GDK_9, ID_FIRSTBUFFER + 8},
+	{ m__M, GDK_0, ID_FIRSTBUFFER + 9},
+	{ 0,   0,		 0 }
+};
+
+gint
+on_anjuta_window_key_press_event (GtkWidget   *widget,
+				  GdkEventKey *event,
+				  gpointer     user_data)
+{
+	int modifiers;
+	int i;
+
+	g_return_val_if_fail (widget != NULL, FALSE);
+	g_return_val_if_fail (GNOME_IS_APP (widget), FALSE);
+	g_return_val_if_fail (event != NULL, FALSE);
+
+	modifiers = event->state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK);
+  
+	for (i = 0; global_keymap[i].id; i++)
+		if (event->keyval == global_keymap[i].gdk_key &&
+		    (event->state & global_keymap[i].modifiers) == global_keymap[i].modifiers)
+			break;
+
+	if (!global_keymap[i].id)
+		return FALSE;
+
+	switch (global_keymap[i].id) {
+	case ID_NEXTBUFFER:
+	case ID_PREVBUFFER: {
+		GtkNotebook *notebook = GTK_NOTEBOOK (app->widgets.notebook);
+		int pages_nb;
+		int cur_page;
+
+		if (!notebook->children)
+			return FALSE;
+
+		if (!g_tabbing)
+		{
+			g_tabbing = TRUE;
+		}
+
+		pages_nb = g_list_length (notebook->children);
+		cur_page = gtk_notebook_get_current_page (notebook);
+
+		if (global_keymap[i].id == ID_NEXTBUFFER)
+			cur_page = (cur_page < pages_nb - 1) ? cur_page + 1 : 0;
+		else
+			cur_page = cur_page ? cur_page - 1 : pages_nb -1;
+
+		gtk_notebook_set_current_page (notebook, cur_page);
+
+		break;
+	}
+	default:
+		if (global_keymap[i].id >= ID_FIRSTBUFFER &&
+		  global_keymap[i].id <= (ID_FIRSTBUFFER + 9))
+		{
+			GtkNotebook *notebook = GTK_NOTEBOOK (app->widgets.notebook);
+			int page_req = global_keymap[i].id - ID_FIRSTBUFFER;
+
+			if (!notebook->children)
+				return FALSE;
+			gtk_notebook_set_current_page (notebook, page_req);
+		}
+		else
+			return FALSE;
+	}
+
+	/* Note: No reason for a shortcut to do more than one thing a time */
+	g_signal_stop_emission_by_name (G_OBJECT (widget), "key_press_event");
+
+	return TRUE;
+}
+
+gint
+on_anjuta_window_key_release_event (GtkWidget   *widget,
+				  GdkEventKey *event,
+				  gpointer     user_data)
+{
+	g_return_val_if_fail (widget != NULL, FALSE);
+	g_return_val_if_fail (GNOME_IS_APP (widget), FALSE);
+	g_return_val_if_fail (event != NULL, FALSE);
+
+	if (g_tabbing && ((event->keyval == GDK_Control_L) ||
+		(event->keyval == GDK_Control_R)))
+	{
+		GtkNotebook *notebook = GTK_NOTEBOOK (app->widgets.notebook);
+		GtkWidget *widget;
+		int cur_page;
+		g_tabbing = FALSE;
+		
+		if (anjuta_preferences_get_int (app->preferences,
+										EDITOR_TABS_RECENT_FIRST))
+		{
+			/*
+			TTimo: move the current notebook page to first position
+			that maintains Ctrl-TABing on a list of most recently edited files
+			*/
+			cur_page = gtk_notebook_get_current_page (notebook);
+			widget = gtk_notebook_get_nth_page (notebook, cur_page);
+			gtk_notebook_reorder_child (notebook, widget, 0);
+		}
+	}
+	return FALSE;
+}
+
 void
-on_open_filesel_ok_clicked (GtkButton * button, gpointer user_data)
+on_open_filesel_ok_clicked (GtkDialog *dlg, gpointer user_data)
 {
 	gchar *full_filename;
 	gchar *entry_filename = NULL;
@@ -180,7 +410,7 @@ on_open_filesel_ok_clicked (GtkButton * button, gpointer user_data)
 	
 	list = fileselection_get_filelist(app->fileselection);
 	elements = g_list_length(list);
-	/* If filename is only only written in entry but not selected (Bug #506441) */
+	/* If filename is only written in entry but not selected (Bug #506441) */
 	if (elements == 0)
 	{
 		entry_filename = fileselection_get_filename(app->fileselection);
@@ -192,37 +422,13 @@ on_open_filesel_ok_clicked (GtkButton * button, gpointer user_data)
 	}
 	for(i=0;i<elements;i++)
 	{
-		/*  full_filename = fileselection_get_filename (app->fileselection); */
-		/*	full_filename = (gchar *)g_list_nth_data(list,i); */
 		full_filename = g_strdup(g_list_nth_data(list,i));
-		/*printf("Filename retrived = %s\n",full_filename);*/
 		if (!full_filename)
 			return;
-		if (strlen (extract_filename (full_filename)) == 0)
-		{
-			g_free (full_filename);
-			return;
-		}
-		if (file_is_regular (full_filename) == FALSE)
-		{
-			anjuta_error (_("Not a regular file: %s."), full_filename);
-			g_free (full_filename);
-			return;
-		}
-		if (file_is_readable (full_filename) == FALSE)
-		{
-			anjuta_error (_("No read permission for: %s."), full_filename);
-			g_free (full_filename);
-			return;
-		}
-		/*printf("I have reached this point\n");*/
 		anjuta_goto_file_line (full_filename, -1);
-		/*printf("I have reached this point goto line\n");*/
-		//gtk_widget_hide (app->fileselection);
-		fileselection_hide_widget(app->fileselection);
 		g_free (full_filename);
 	}
-	fileselection_hide_widget(app->fileselection);
+
 	if (entry_filename)
 	{
 		g_list_remove(list, entry_filename);
@@ -231,39 +437,8 @@ on_open_filesel_ok_clicked (GtkButton * button, gpointer user_data)
 	/* g_free(list); */
 }
 
-void
-on_open_filesel_cancel_clicked (GtkButton * button, gpointer user_data)
-{
-	fileselection_hide_widget(app->fileselection);
-	//gtk_widget_hide (app->fileselection);
-}
-
-void
-on_save_as_filesel_ok_clicked (GtkButton * button, gpointer user_data)
-{
-	gchar *filename, *buff;
-
-	filename = fileselection_get_filename (app->save_as_fileselection);
-	if (file_is_regular (filename))
-	{
-		buff =
-			g_strdup_printf (_
-					 ("The file \"%s\" already exists.\nDo you want to overwrite it?."),
-					 filename);
-		messagebox2 (GNOME_MESSAGE_BOX_QUESTION, buff,
-			     GNOME_STOCK_BUTTON_YES, GNOME_STOCK_BUTTON_NO,
-			     GTK_SIGNAL_FUNC
-			     (on_save_as_overwrite_yes_clicked), NULL,
-			     user_data);
-		g_free (buff);
-	}
-	else
-		on_save_as_overwrite_yes_clicked (NULL, user_data);
-	g_free (filename);
-}
-
-void
-on_save_as_overwrite_yes_clicked (GtkButton * button, gpointer user_data)
+static void
+save_as_real (void)
 {
 	TextEditor *te;
 	gchar *full_filename, *saved_filename, *saved_full_filename;
@@ -284,7 +459,7 @@ on_save_as_overwrite_yes_clicked (GtkButton * button, gpointer user_data)
 	
 	te->full_filename = g_strdup (full_filename);
 	te->filename = g_strdup (extract_filename (full_filename));
-	status = text_editor_save_file (te);
+	status = text_editor_save_file (te, TRUE);
 	gtk_widget_hide (app->save_as_fileselection);
 	if (status == FALSE)
 	{
@@ -338,168 +513,55 @@ on_save_as_overwrite_yes_clicked (GtkButton * button, gpointer user_data)
 			}
 		}
 		anjuta_update_title ();
-		anjuta_clear_windows_menu();
-		anjuta_fill_windows_menu();
 		g_free (full_filename);
 	}
 }
 
-gboolean
-on_anjuta_window_focus_in_event (GtkWidget * widget,
-				 GdkEventFocus * event, gpointer user_data)
+void
+on_save_as_filesel_ok_clicked (GtkDialog *dlg, gpointer user_data)
 {
-	gint page_num;
-	TextEditor *te;
+	gchar *filename;
 
-	if (g_list_length (GTK_NOTEBOOK (app->widgets.notebook)->children) >
-	    0)
+	filename = fileselection_get_filename (app->save_as_fileselection);
+	if (file_is_regular (filename))
 	{
-		page_num =
-			gtk_notebook_get_current_page (GTK_NOTEBOOK
-						       (app->widgets.
-							notebook));
-		if (page_num >= 0)
-			te = anjuta_get_notebook_text_editor (page_num);
-		else
-			te = NULL;
-	}
-	else if (app->text_editor_list == NULL)
-	{
-		te = NULL;
+		GtkWidget *dialog;
+		dialog = gtk_message_dialog_new (GTK_WINDOW (app->widgets.window),
+										 GTK_DIALOG_DESTROY_WITH_PARENT,
+										 GTK_MESSAGE_QUESTION,
+										 GTK_BUTTONS_NONE,
+										 _("The file '%s' already exists.\n"
+										 "Do you want to replace it with the one you are saving?"),
+										 filename);
+		gtk_dialog_add_button (GTK_DIALOG(dialog),
+							   GTK_STOCK_CANCEL,
+							   GTK_RESPONSE_CANCEL);
+		anjuta_dialog_add_button (GTK_DIALOG (dialog),
+								  _("_Replace"),
+								  GTK_STOCK_REFRESH,
+								  GTK_RESPONSE_YES);
+		if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES)
+			save_as_real ();
+		gtk_widget_destroy (dialog);
 	}
 	else
-	{
-		/* If there is TextEditor in the list, but no page 
-		 * * in the notebook, we don't change the current
-		 * * TextEditor
-		 */
-		return FALSE;
-	}
-	if (te == anjuta_get_current_text_editor ())
-		return FALSE;
-	anjuta_set_current_text_editor (te);
-	return FALSE;
-}
+		save_as_real ();
+	g_free (filename);
 
-enum {
-	m__ = 0,
-	mS_ = GDK_SHIFT_MASK,
-	m_C = GDK_CONTROL_MASK,
-	mSC = GDK_SHIFT_MASK | GDK_CONTROL_MASK
-};
-
-enum {
-	ID_NEXTBUFFER = 1, /* Note: the value mustn't be 0 ! */
-	ID_PREVBUFFER
-};
-
-typedef struct {
-	int modifiers;
-	unsigned int gdk_key;
-	int id;
-} ShortcutMapping;
-
-static ShortcutMapping global_keymap[] = {
-	{ m_C, GDK_Tab,		 ID_NEXTBUFFER },
-	{ mSC, GDK_ISO_Left_Tab, ID_PREVBUFFER },
-	{ 0,   0,		 0 }
-};
-
-gint
-on_anjuta_window_key_press_event (GtkWidget   *widget,
-				  GdkEventKey *event,
-				  gpointer     user_data)
-{
-	int modifiers;
-	int i;
-
-	g_return_val_if_fail (widget != NULL, FALSE);
-	g_return_val_if_fail (GNOME_IS_APP (widget), FALSE);
-	g_return_val_if_fail (event != NULL, FALSE);
-
-	modifiers = event->state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK);
-
-	for (i = 0; global_keymap[i].id; i++)
-		if (event->keyval == global_keymap[i].gdk_key &&
-		    (event->state & global_keymap[i].modifiers) == global_keymap[i].modifiers)
-			break;
-
-	if (!global_keymap[i].id)
-		return FALSE;
-
-	switch (global_keymap[i].id) {
-	case ID_NEXTBUFFER:
-	case ID_PREVBUFFER: {
-		GtkNotebook *notebook = GTK_NOTEBOOK (app->widgets.notebook);
-		int pages_nb;
-		int cur_page;
-
-		if (!notebook->children)
-			return FALSE;
-
-		pages_nb = g_list_length (notebook->children);
-		cur_page = gtk_notebook_get_current_page (notebook);
-
-		if (global_keymap[i].id == ID_NEXTBUFFER)
-			cur_page = (cur_page < pages_nb - 1) ? cur_page + 1 : 0;
-		else
-			cur_page = cur_page ? cur_page - 1 : pages_nb -1;
-
-		gtk_notebook_set_page (notebook, cur_page);
-
-		break;
-	}
-	default:
-		return FALSE;
-	}
-
-	/* Note: No reason for a shortcut to do more than one thing a time */
-	gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");
-
-	return TRUE;
+	if (anjuta_preferences_get_int (ANJUTA_PREFERENCES (app->preferences),
+									EDITOR_TABS_ORDERING))
+		anjuta_order_tabs ();
 }
 
 void
-on_save_as_filesel_cancel_clicked (GtkButton * button, gpointer user_data)
+on_save_as_filesel_cancel_clicked (GtkDialog *dlg, gpointer user_data)
 {
 	gtk_widget_hide (app->save_as_fileselection);
 	closing_state = FALSE;
 }
 
-void
-on_build_msg_save_ok_clicked(GtkButton * button, gpointer user_data)
-{
-	gchar *filename, *buff;
-
-	filename = fileselection_get_filename (app->save_as_build_msg_sel);
-	if (file_is_regular (filename))
-	{
-		buff =
-			g_strdup_printf (_
-					 ("The file \"%s\" already exists.\nDo you want to overwrite it?."),
-					 filename);
-		messagebox2 (GNOME_MESSAGE_BOX_QUESTION, buff,
-			     GNOME_STOCK_BUTTON_YES, GNOME_STOCK_BUTTON_NO,
-			     GTK_SIGNAL_FUNC
-			     (on_build_msg_save_overwrite), NULL,
-			     user_data);
-		g_free (buff);
-	}
-	else
-		on_build_msg_save_overwrite(NULL, user_data);
-	g_free (filename);
-}
-
-void
-on_build_msg_save_cancel_clicked(GtkButton * button, gpointer user_data)
-{
-	gtk_widget_hide (app->save_as_build_msg_sel);
-	closing_state = FALSE;
-}
-
-
-void 
-on_build_msg_save_overwrite(GtkButton * button, gpointer user_data)
+static void
+build_msg_save_real (void)
 {
 	gchar *filename;
 	FILE *msgfile; 
@@ -508,17 +570,55 @@ on_build_msg_save_overwrite(GtkButton * button, gpointer user_data)
 	msgfile = fopen(filename, "w");
 	if (! msgfile)
 	{
-		anjuta_error("Could not open file for writing");
+		anjuta_error (_("Could not open file for writing"));
 		return;
 	}
 	
-	anjuta_message_manager_save_build(app->messages, msgfile);
+	an_message_manager_save_build(app->messages, msgfile);
 			
 	fclose(msgfile);
 
 	return;
 }
 
+void
+on_build_msg_save_ok_clicked(GtkDialog *dlg, gpointer user_data)
+{
+	gchar *filename;
+
+	filename = fileselection_get_filename (app->save_as_build_msg_sel);
+	if (file_is_regular (filename))
+	{
+		GtkWidget *dialog;
+		dialog = gtk_message_dialog_new (GTK_WINDOW (app->widgets.window),
+										 GTK_DIALOG_DESTROY_WITH_PARENT,
+										 GTK_MESSAGE_QUESTION,
+										 GTK_BUTTONS_NONE,
+										 _("The file '%s' already exists.\n"
+										 "Do you want to replace it with the one you are saving?."),
+										 filename);
+		gtk_dialog_add_button (GTK_DIALOG (dialog),
+							   GTK_STOCK_CANCEL,
+							   GTK_RESPONSE_CANCEL);
+		anjuta_dialog_add_button (GTK_DIALOG (dialog),
+								  _("_Replace"),
+								  GTK_STOCK_REFRESH,
+								  GTK_RESPONSE_YES);
+		if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES)
+			build_msg_save_real ();
+		gtk_widget_destroy (dialog);
+	}
+	else
+		build_msg_save_real ();
+	g_free (filename);
+}
+
+void
+on_build_msg_save_cancel_clicked(GtkDialog *dlg, gpointer user_data)
+{
+	gtk_widget_hide (app->save_as_build_msg_sel);
+	closing_state = FALSE;
+}
 
 void
 on_prj_list_undock_button_clicked (GtkButton * button, gpointer user_data)
@@ -544,7 +644,7 @@ on_mesg_win_hide_button_clicked (GtkButton * button, gpointer user_data)
 void
 on_mesg_win_undock_button_clicked (GtkButton * button, gpointer user_data)
 {
-	anjuta_message_manager_undock (app->messages);
+	an_message_manager_undock (app->messages);
 }
 
 void
