@@ -54,32 +54,42 @@ on_need_term_checkbutton_toggled (GtkToggleButton * tb, gpointer user_data)
 	aw->need_terminal = gtk_toggle_button_get_active (tb);
 }
 
+static void
+on_use_glade_checkbutton_toggled (GtkToggleButton * tb, gpointer user_data)
+{
+	AppWizard *aw = user_data;
+	aw->use_glade = gtk_toggle_button_get_active (tb);
+}
+
 void
 create_app_wizard_page4 (AppWizard * aw)
 {
-	gtk_signal_connect (GTK_OBJECT (aw->widgets.file_header_check), "toggled",
-			    GTK_SIGNAL_FUNC (on_gpl_checkbutton_toggled), aw);
-	gtk_signal_connect (GTK_OBJECT (aw->widgets.gettext_support_check), "toggled",
-			    GTK_SIGNAL_FUNC(on_gettext_support_checkbutton_toggled), aw);
+	g_signal_connect (G_OBJECT (aw->widgets.file_header_check), "toggled",
+			    G_CALLBACK (on_gpl_checkbutton_toggled), aw);
+	g_signal_connect (G_OBJECT (aw->widgets.gettext_support_check), "toggled",
+			    G_CALLBACK(on_gettext_support_checkbutton_toggled), aw);
 
-	gtk_signal_connect (GTK_OBJECT (aw->widgets.menu_entry_entry), "changed",
-			    GTK_SIGNAL_FUNC (on_aw_text_entry_changed), &aw->menu_entry);
-	gtk_signal_connect (GTK_OBJECT (aw->widgets.menu_entry_entry), "realize",
-			    GTK_SIGNAL_FUNC (on_aw_text_entry_realize), aw->menu_entry);
+	g_signal_connect (G_OBJECT (aw->widgets.menu_entry_entry), "changed",
+			    G_CALLBACK (on_aw_text_entry_changed), &aw->menu_entry);
+	g_signal_connect (G_OBJECT (aw->widgets.menu_entry_entry), "realize",
+			    G_CALLBACK (on_aw_text_entry_realize), aw->menu_entry);
 
-	gtk_signal_connect (GTK_OBJECT (aw->widgets.menu_comment_entry), "changed",
-			    GTK_SIGNAL_FUNC (on_aw_text_entry_changed), &aw->menu_comment);
-	gtk_signal_connect (GTK_OBJECT (aw->widgets.menu_comment_entry), "realize",
-			    GTK_SIGNAL_FUNC (on_aw_text_entry_realize), aw->menu_comment);
+	g_signal_connect (G_OBJECT (aw->widgets.menu_comment_entry), "changed",
+			    G_CALLBACK (on_aw_text_entry_changed), &aw->menu_comment);
+	g_signal_connect (G_OBJECT (aw->widgets.menu_comment_entry), "realize",
+			    G_CALLBACK (on_aw_text_entry_realize), aw->menu_comment);
 
-	gtk_signal_connect (GTK_OBJECT (aw->widgets.app_group_combo), "changed",
-			    GTK_SIGNAL_FUNC (on_aw_text_entry_changed), &aw->app_group);
+	g_signal_connect (G_OBJECT (aw->widgets.app_group_combo), "changed",
+			    G_CALLBACK (on_aw_text_entry_changed), &aw->app_group);
 	
-	gtk_signal_connect (GTK_OBJECT (aw->widgets.term_check), "toggled",
-			    GTK_SIGNAL_FUNC(on_need_term_checkbutton_toggled), aw);
+	g_signal_connect (G_OBJECT (aw->widgets.term_check), "toggled",
+			    G_CALLBACK(on_need_term_checkbutton_toggled), aw);
+	g_signal_connect (G_OBJECT (aw->widgets.use_glade_check), "toggled",
+			    G_CALLBACK(on_use_glade_checkbutton_toggled), aw);
 
 	gtk_widget_ref (aw->widgets.gettext_support_check);
 	gtk_widget_ref (aw->widgets.file_header_check);
+	gtk_widget_ref (aw->widgets.use_glade_check);
 	gtk_widget_ref (aw->widgets.menu_frame);
 	gtk_widget_ref (aw->widgets.menu_entry_entry);
 	gtk_widget_ref (aw->widgets.menu_comment_entry);
