@@ -28,7 +28,6 @@
 #include <gnome.h>
 
 #include "anjuta.h"
-#include "messagebox.h"
 #include "resources.h"
 #include "appwizard.h"
 
@@ -40,18 +39,25 @@ app_wizard_new (void)
 	{
 		gchar* username;
 		
-		username = getenv ("USERNAME");
+		username =
+			anjuta_preferences_get (ANJUTA_PREFERENCES (app->preferences),
+									IDENT_NAME);
 		if (!username)
-			username = getenv ("USER");
-		
+		{
+			username = getenv ("USERNAME");
+			if (!username)
+				username = getenv ("USER");
+			if (!username)
+				username = "Unknown";
+			username = g_strdup(username);
+		}
 		aw->prj_type = PROJECT_TYPE_GNOME;
+		aw->active_project_types = NULL;
 		aw->target_type = PROJECT_TARGET_TYPE_EXECUTABLE;
 		aw->prj_name = NULL;
 		aw->target = NULL;
-		if (username)
-			aw->author = g_strdup (username);
-		else
-			aw->author = NULL;
+		aw->author = g_strdup (username);
+		g_free(username);
 		aw->version = g_strdup ("0.1");
 		aw->description = NULL;
 		aw->gettext_support = TRUE;
@@ -63,7 +69,8 @@ app_wizard_new (void)
 		aw->app_group = g_strdup ("Applications");
 		aw->use_header = TRUE;
 		aw->language = PROJECT_PROGRAMMING_LANGUAGE_C;
-
+		aw->use_glade = TRUE;
+		
 		create_app_wizard_gui (aw);
 		create_app_wizard_page1 (aw);
 		create_app_wizard_page2 (aw);
@@ -89,7 +96,7 @@ app_wizard_destroy (AppWizard * aw)
 	gint i;
 	
 	g_return_if_fail (aw != NULL);
-
+	g_list_free (aw->active_project_types);
 	gtk_widget_unref (aw->widgets.window);
 	gtk_widget_unref (aw->widgets.druid);
 	for (i=0; i<6; i++)
@@ -110,15 +117,16 @@ app_wizard_destroy (AppWizard * aw)
 	gtk_widget_unref (aw->widgets.app_group_combo);
 	gtk_widget_unref (aw->widgets.app_group_entry);
 	gtk_widget_unref (aw->widgets.term_check);
+	gtk_widget_unref (aw->widgets.use_glade_check);
 
 	gtk_widget_unref (aw->widgets.language_c_radio);
 	gtk_widget_unref (aw->widgets.language_cpp_radio);
 	gtk_widget_unref (aw->widgets.language_c_cpp_radio);
-#if 0
+	
 	gtk_widget_unref (aw->widgets.target_exec_radio);
 	gtk_widget_unref (aw->widgets.target_slib_radio);
 	gtk_widget_unref (aw->widgets.target_dlib_radio);
-#endif
+
 	if (GTK_IS_WIDGET (aw->widgets.window))
 		gtk_widget_destroy (aw->widgets.window);
 	string_assign (&aw->prj_name, NULL);
