@@ -21,5 +21,7 @@
 #define _BUILD_FILE_H_
 
 void build_file(void);
+gboolean build_execute_command (const gchar *command);
+gboolean build_execute_shell_command (const gchar *command);
 
 #endif
