@@ -20,8 +20,5 @@
 #define _COMPILE_H_
 
 void compile_file(gboolean use_make);
-void compile_mesg_arrived(gchar* mesg);
-void compile_terminated(int status, time_t time);
 
 #endif
-
