@@ -27,12 +27,14 @@ struct _FileSelData
 {
   char title[MAX_NAME_LENGTH];
   gpointer data;
-  void (*click_ok_callback)(GtkButton*, gpointer);
-  void (*click_cancel_callback)(GtkButton*, gpointer);
+  void (*click_ok_callback)(GtkDialog*, gpointer);
+  void (*click_cancel_callback)(GtkDialog*, gpointer);
   GtkWidget *filesel;
   
 };
   
+void fileselection_update_dir(GtkWidget *filesel);
+
 GtkWidget* create_fileselection_gui (FileSelData *fd);
 
 /* Free the return */
@@ -53,13 +55,25 @@ fileselection_set_filename (GtkWidget* filesel, gchar* fname);
 GList *
 fileselection_get_filelist(GtkWidget * filesel);
 
-GList *
-fileselection_get_nodelist(GtkWidget * filesel);
-
-gchar * 
-fileselection_get_lastfilename(GtkWidget * widget, GList * list);
-
 void 
 fileselection_hide_widget(GtkWidget * widget);
 
+GtkWidget * 
+fileselection_storetypes(GtkWidget *filesel, GList *filetypes);
+
+GList * 
+fileselection_addtype_f(GList *filetypes, gchar *description, ...);
+
+GList * 
+fileselection_addtype(GList *filetypes, gchar *description, GList *extentions);
+
+GList * 
+fileselection_getcombolist(GtkWidget * filesel, GList *filetypes);
+
+void 
+fileselection_set_combolist(GtkWidget* filesel, GList *combolist);
+
+GtkWidget*
+fileselection_clearfiletypes(GtkWidget* filesel);
+
 #endif
