@@ -27,10 +27,8 @@ typedef struct _FormatSubMenu FormatSubMenu;
 typedef struct _BuildSubMenu BuildSubMenu;
 typedef struct _BookmarkSubMenu BookmarkSubMenu;
 typedef struct _DebugSubMenu DebugSubMenu;
-typedef struct _UtilitiesSubMenu UtilitiesSubMenu;
-typedef struct _WindowsSubMenu WindowsSubMenu;
+typedef struct _CVSSubMenu CVSSubMenu;
 typedef struct _SettingsSubMenu SettingsSubMenu;
-typedef struct _HelpSubMenu HelpSubMenu;
 typedef struct _MainMenuBar MainMenuBar;
 
 struct _FileSubMenu
@@ -49,8 +47,8 @@ struct _FileSubMenu
 	GtkWidget *save_project;
 	GtkWidget *close_project;
 	GtkWidget *rename;
-	GtkWidget *page_setup;
 	GtkWidget *print;
+	GtkWidget *print_preview;
 	GtkWidget *recent_files;
 	GtkWidget *recent_projects;
 	GtkWidget *exit;
@@ -71,6 +69,22 @@ struct _EditSubMenu
 	GtkWidget *convert_lf;
 	GtkWidget *convert_cr;
 	GtkWidget *convert_auto;
+
+	GtkWidget *insert_header;
+	GtkWidget *insert_custom_indent;
+	GtkWidget *insert_c_switch;
+	GtkWidget *insert_c_for;
+	GtkWidget *insert_c_while;
+	GtkWidget *insert_c_ifelse;
+	
+	GtkWidget *insert_cvs_author;
+	GtkWidget *insert_cvs_date;
+	GtkWidget *insert_cvs_header;
+	GtkWidget *insert_cvs_id;
+	GtkWidget *insert_cvs_log;
+	GtkWidget *insert_cvs_name;
+	GtkWidget *insert_cvs_revision;
+	GtkWidget *insert_cvs_source;
 	
 	GtkWidget *insert_c_gpl;
 	GtkWidget *insert_cpp_gpl;
@@ -78,18 +92,28 @@ struct _EditSubMenu
 	GtkWidget *insert_username;
 	GtkWidget *insert_datetime;
 	GtkWidget *insert_header_template;
+	GtkWidget *insert_changelog_entry;
 
 	GtkWidget *select_all;
 	GtkWidget *select_brace;
 	GtkWidget *select_block;
-
+	GtkWidget *select_function;
+	GtkWidget *select_line;
+	GtkWidget *select_word;
+	
 	GtkWidget *autocomplete;
 	GtkWidget *calltip;
 
 	GtkWidget *find;
-	GtkWidget *repeat_find;	
+	GtkWidget *find_next;	
+	GtkWidget *find_previous;
 	GtkWidget *find_in_files;
 	GtkWidget *find_replace;
+	GtkWidget *enter_selection;
+
+	GtkWidget *comment_block;
+	GtkWidget *comment_box;
+	GtkWidget *comment_stream;
 
 	GtkWidget *goto_line;
 	GtkWidget *goto_brace;
@@ -97,8 +121,10 @@ struct _EditSubMenu
 	GtkWidget *goto_block_end;
 	GtkWidget *goto_prev_mesg;
 	GtkWidget *goto_next_mesg;
-
-	GtkWidget *edit_app_gui;
+	GtkWidget *go_back;
+	GtkWidget *go_forward;
+	GtkWidget *goto_tag_def;
+	GtkWidget *goto_tag_decl;
 };
 
 struct _ViewSubMenu
@@ -115,38 +141,31 @@ struct _ViewSubMenu
 	GtkWidget *editor_indentguides;
 	GtkWidget *editor_whitespaces;
 	GtkWidget *editor_eolchars;
+	GtkWidget *editor_linewrap;
 
 	GtkWidget *messages;
 	GtkWidget *project_listing;
 	GtkWidget *bookmarks;
 	GtkWidget *breakpoints;
-	GtkWidget *variable_watch;
 	GtkWidget *registers;
-	GtkWidget *program_stack;
 	GtkWidget *shared_lib;
 	GtkWidget *signals;
 	GtkWidget *dump_window;
-	GtkWidget *console;
-	GtkWidget *show_hide_locals;
 };
 
 struct _ProjectSubMenu
 {
-	GtkWidget *new_file;
-	GtkWidget *add_inc;
-	GtkWidget *add_src;
-	GtkWidget *add_hlp;
-	GtkWidget *add_data;
-	GtkWidget *add_pix;
-	GtkWidget *add_po;
-	GtkWidget *add_doc;
-	GtkWidget *remove;
-	GtkWidget *readme;
-	GtkWidget *todo;
-	GtkWidget *changelog;
-	GtkWidget *news;
+	GtkWidget *add_file;
+	GtkWidget *view_file;
+	GtkWidget *edit_file;
+	GtkWidget *remove_file;
 	GtkWidget *configure;
-	GtkWidget *info;
+	GtkWidget *project_info;
+	GtkWidget *dock_undock;
+	GtkWidget *update_tags;
+	GtkWidget *rebuild_tags;
+	GtkWidget *edit_app_gui;
+	GtkWidget *project_help;
 };
 
 struct _FormatSubMenu
@@ -155,7 +174,6 @@ struct _FormatSubMenu
 	GtkWidget *indent_inc;
 	GtkWidget *indent_dcr;
 	GtkWidget *force_hilite;
-	GtkWidget *update_tags;
 	GtkWidget *open_folds;
 	GtkWidget *close_folds;
 	GtkWidget *toggle_fold;
@@ -168,6 +186,7 @@ struct _BuildSubMenu
 	GtkWidget *make;
 	GtkWidget *build;
 	GtkWidget *build_all;
+	GtkWidget *save_build_messages;
 	GtkWidget *install;
 	GtkWidget *autogen;
 	GtkWidget *configure;
@@ -222,25 +241,25 @@ struct _DebugSubMenu
 	GtkWidget *info_locals;
 	GtkWidget *info_frame;
 	GtkWidget *info_args;
+	GtkWidget *info_memory;
 };
 
-struct _UtilitiesSubMenu
+struct _CVSSubMenu
 {
-	GtkWidget *grep;
-	GtkWidget *compare;
-	GtkWidget *diff;
-	GtkWidget *view;
-	GtkWidget *indent;
-	GtkWidget *flow;
-	GtkWidget *cross_ref;
-	GtkWidget *trace;
-	GtkWidget *archive;
-};
-
-struct _WindowsSubMenu
-{
-	GtkWidget *new;
-	GtkWidget *close;
+	GtkWidget* update_file;
+	GtkWidget* commit_file;
+	GtkWidget* add_file;
+	GtkWidget* remove_file;
+	GtkWidget* status_file;
+	GtkWidget* log_file;
+	GtkWidget* diff_file;
+	GtkWidget* update_project;
+	GtkWidget* commit_project;
+	GtkWidget* import_project;
+	GtkWidget* status_project;
+	GtkWidget* log_project;
+	GtkWidget* diff_project;
+	GtkWidget* login;
 };
 
 struct _SettingsSubMenu
@@ -249,9 +268,17 @@ struct _SettingsSubMenu
 	GtkWidget *src_paths;
 	GtkWidget *commands;
 	GtkWidget *preferences;
+	GtkWidget *style_editor;
+	GtkWidget *file_view_filters;
+	GtkWidget *user_properties;
 	GtkWidget *default_preferences;
+	GtkWidget *shortcuts;
 };
 
+/* Help menu should not be touched as it
+is often modifed by gnome when the help
+TOC (table of contents) is not found, resulting a screw-up */
+/*
 struct _HelpSubMenu
 {
 	GtkWidget *gnome;
@@ -261,7 +288,7 @@ struct _HelpSubMenu
 	GtkWidget *search;
 	GtkWidget *about;
 };
-
+*/
 struct _MainMenuBar
 {
 	FileSubMenu file;
@@ -272,10 +299,9 @@ struct _MainMenuBar
 	BuildSubMenu build;
 	BookmarkSubMenu bookmark;
 	DebugSubMenu debug;
-	UtilitiesSubMenu utilities;
-	WindowsSubMenu windows;
+	CVSSubMenu cvs;
 	SettingsSubMenu settings;
-	HelpSubMenu help;
+	/* HelpSubMenu help; */
 };
 
 void create_main_menubar (GtkWidget * app, MainMenuBar * mb);
@@ -283,7 +309,6 @@ void create_main_menubar (GtkWidget * app, MainMenuBar * mb);
 GtkWidget *create_submenu (gchar * title, GList * strings,
 			   GtkSignalFunc callback_func);
 
-
-void main_menu_install_hints (void);
+void main_menu_install_hints (GtkWidget* gnome_app);
 
 #endif
