@@ -17,6 +17,14 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
+#include <assert.h>
+#include <glib-object.h>
+#include <gnome.h>
+
 #include "message-manager.h"
 #include "message-manager-private.h"
 #include "message-manager-dock.h"
@@ -25,116 +33,105 @@
 
 extern "C"
 {
+#include "anjuta.h"
 #include "utilities.h"
 };
 
-// SIGNALS
-
-enum
-{
-	MESSAGE_CLICKED,
-	SIGNALS_END
-};
-
-static guint anjuta_message_manager_signals[SIGNALS_END] = { 0 };
-
 // Data:
 static char *labels[] =
-	{ _("Build"), _("Debug"), _("Find"), _("CVS"), _("Locals"),
-_("Terminal") };
+{
+	N_("Build"),
+	N_("Find"),
+	N_("CVS"),
+	N_("Terminal"),
+	N_("Stdout"),
+	N_("Stderr"),
+	N_("Debug"),
+	N_("Locals"),
+	N_("Watches"),
+	N_("Stack"),
+};
 
 // Intern functions
-static void anjuta_message_manager_destroy (GtkObject * object);
-static void anjuta_message_manager_class_init (AnjutaMessageManagerClass *
+static void an_message_manager_finalize (GObject * object);
+static void an_message_manager_class_init (AnMessageManagerClass *
 					       klass);
-static void anjuta_message_manager_init (GtkObject * obj);
-static void anjuta_message_manager_show_impl (GtkWidget * widget);
-static void anjuta_message_manager_hide_impl (GtkWidget * widget);
+static void an_message_manager_init (GtkObject * obj);
+static void an_message_manager_show_impl (GtkWidget * widget);
+static void an_message_manager_hide_impl (GtkWidget * widget);
 
 // Callbacks
 static gboolean on_dock_activate (GtkWidget * menuitem,
-				  AnjutaMessageManager * amm);
+				  AnMessageManager * amm);
 static gboolean on_show_hide_tab (GtkWidget * menuitem,
 				  MessageSubwindow * msg_win);
-static gboolean on_popup_clicked (AnjutaMessageManager * amm,
-				  GdkEvent * event);
-static void on_msg_double_clicked (GtkCList * list, gint row, gint column,
-				   GdkEvent * event, gpointer msg_win);
-
+static gboolean
+on_popup_clicked (GtkWidget* widget, GdkEvent * event, 
+	AnMessageManagerPrivate* intern);
 // Intern functions:
 
 GtkFrameClass *parent_class;
 
 GtkWidget *
-anjuta_message_manager_new ()
+an_message_manager_new ()
 {
 	GtkWidget *amm;
-	amm = GTK_WIDGET (gtk_type_new (anjuta_message_manager_get_type ()));
+	amm = GTK_WIDGET (g_object_new (AN_MESSAGE_MANAGER_TYPE, NULL));
 	return amm;
 }
 
-GtkType
-anjuta_message_manager_get_type (void)
+guint
+an_message_manager_get_type (void)
 {
-	static GtkType type = 0;
+	static GType type = 0;
 
 	if (!type)
 	{
-		static const GtkTypeInfo info = {
-			"AnjutaMessageManager",
-			sizeof (AnjutaMessageManager),
-			sizeof (AnjutaMessageManagerClass),
-			(GtkClassInitFunc) anjuta_message_manager_class_init,
-			(GtkObjectInitFunc) anjuta_message_manager_init,
-			NULL,
-			NULL,
-			(GtkClassInitFunc) NULL
+		static const GTypeInfo info = 
+		{
+			sizeof (AnMessageManagerClass),
+			(GBaseInitFunc) NULL,
+			(GBaseFinalizeFunc) NULL,
+			(GClassInitFunc) an_message_manager_class_init,
+			(GClassFinalizeFunc) NULL,
+			NULL,           /* class_data */
+			sizeof (AnMessageManager),
+			0,              /* n_preallocs */
+			(GInstanceInitFunc) an_message_manager_init,
+			NULL            /* value_table */
 		};
-		type = gtk_type_unique (gtk_frame_get_type (), &info);
+		type = g_type_register_static (GTK_TYPE_FRAME,
+									   "AnMessageManager",
+									   &info, (GTypeFlags)0);
 	}
 	return type;
 }
 
 static void
-anjuta_message_manager_class_init (AnjutaMessageManagerClass * klass)
+an_message_manager_class_init (AnMessageManagerClass * klass)
 {
-	GtkObjectClass *object_class =
-		reinterpret_cast < GtkObjectClass * >(klass);
-
-	parent_class = reinterpret_cast < GtkFrameClass * >(klass);
-	// Signals
-	anjuta_message_manager_signals[MESSAGE_CLICKED] =
-		gtk_signal_new ("message_clicked", GTK_RUN_FIRST,
-				object_class->type,
-				GTK_SIGNAL_OFFSET (AnjutaMessageManagerClass,
-						   message_clicked),
-				gtk_marshal_NONE__POINTER, GTK_TYPE_NONE,
-				1, GTK_TYPE_POINTER);
-
-	gtk_object_class_add_signals (object_class,
-				      anjuta_message_manager_signals,
-				      SIGNALS_END);
-
-	object_class->destroy = anjuta_message_manager_destroy;
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	parent_class = reinterpret_cast < GtkFrameClass * >
+						(g_type_class_peek_parent (klass));
+	object_class->finalize = an_message_manager_finalize;
 }
 
 static void
-anjuta_message_manager_init (GtkObject * obj)
+an_message_manager_init (GtkObject * obj)
 {
 	// Init Class
-	AnjutaMessageManager *amm = ANJUTA_MESSAGE_MANAGER (obj);
-	gtk_widget_set_usize (GTK_WIDGET (amm), 200, 70);
+	AnMessageManager *amm = AN_MESSAGE_MANAGER (obj);
 
-	amm->intern = new AnjutaMessageManagerPrivate;
+	amm->intern = new AnMessageManagerPrivate;
 	amm->intern->last_page = -1;
 	amm->intern->cur_msg_win = 0;
 
 	// Create Widgets
 	amm->intern->notebook = gtk_notebook_new ();
 	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (amm->intern->notebook),
-				  GTK_POS_LEFT);
+				  			  GTK_POS_BOTTOM);
 	gtk_notebook_set_scrollable (GTK_NOTEBOOK (amm->intern->notebook),
-				     true);
+				     			 true);
 	gtk_widget_show (amm->intern->notebook);
 
 	amm->intern->window = NULL;
@@ -143,8 +140,9 @@ anjuta_message_manager_init (GtkObject * obj)
 	static GtkItemFactoryEntry menu_items[] = {
 		{"/separator", NULL, NULL, 0, "<Separator>"}
 	};
-	GtkWidget* dock_item = gtk_menu_item_new_with_label(_("Dock/Undock"));
-	gtk_signal_connect(GTK_OBJECT(dock_item), "activate", GTK_SIGNAL_FUNC(on_dock_activate), amm);
+	GtkWidget* dock_item = gtk_check_menu_item_new_with_label(_("Docked"));
+	g_signal_connect(GTK_OBJECT(dock_item), "activate",
+					   GTK_SIGNAL_FUNC(on_dock_activate), amm);
 	gtk_widget_show(dock_item);
 	
 	GtkAccelGroup *agroup = gtk_accel_group_new ();
@@ -153,21 +151,22 @@ anjuta_message_manager_init (GtkObject * obj)
 	gtk_item_factory_create_items (factory, 1, menu_items, NULL);
 	amm->intern->popupmenu =
 		gtk_item_factory_get_widget (factory, "<none>");
-	gtk_menu_prepend(GTK_MENU(amm->intern->popupmenu), dock_item);
-	
-	gtk_signal_connect (GTK_OBJECT (amm), "button_press_event",
-			    GTK_SIGNAL_FUNC (on_popup_clicked), amm);
+	gtk_menu_shell_prepend(GTK_MENU_SHELL(amm->intern->popupmenu), dock_item);
+	amm->intern->dock_item = dock_item;
+	g_object_ref(G_OBJECT(amm->intern->dock_item));
+	g_signal_connect (GTK_WIDGET (amm), "button_press_event",
+			    GTK_SIGNAL_FUNC (on_popup_clicked), amm->intern);
 
 	// Add Widgets to UI
 	gtk_container_add (GTK_CONTAINER (amm), amm->intern->notebook);
 
-	gtk_signal_connect (GTK_OBJECT (amm), "show",
+	g_signal_connect (GTK_OBJECT (amm), "show",
 			    GTK_SIGNAL_FUNC
-			    (anjuta_message_manager_show_impl),
+			    (an_message_manager_show_impl),
 			    GTK_WIDGET (amm));
-	gtk_signal_connect (GTK_OBJECT (amm), "hide",
+	g_signal_connect (GTK_OBJECT (amm), "hide",
 			    GTK_SIGNAL_FUNC
-			    (anjuta_message_manager_hide_impl),
+			    (an_message_manager_hide_impl),
 			    GTK_WIDGET (amm));
 
 	amm->intern->is_docked = false;
@@ -175,23 +174,23 @@ anjuta_message_manager_init (GtkObject * obj)
 }
 
 static void
-anjuta_message_manager_destroy (GtkObject * obj)
+an_message_manager_finalize (GObject * obj)
 {
-	AnjutaMessageManager *amm = ANJUTA_MESSAGE_MANAGER (obj);
+	AnMessageManager *amm = AN_MESSAGE_MANAGER (obj);
 	typedef vector < MessageSubwindow * >::iterator I;
 	for (I cur_win = amm->intern->msg_windows.begin ();
 	     cur_win != amm->intern->msg_windows.end (); cur_win++)
 	{
 		delete *cur_win;
 	}
-	GTK_OBJECT_CLASS (parent_class)->destroy (obj);
+	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (obj));
 }
 
 // Public:
 
 // Creates a new Notebook page for message of a new type if it not already exists
 gboolean
-anjuta_message_manager_add_type (AnjutaMessageManager * amm, gint type_name,
+an_message_manager_add_type (AnMessageManager * amm, gint type_name,
 				 const gchar * pixmap)
 {
 	if (!amm || !pixmap)
@@ -208,23 +207,25 @@ anjuta_message_manager_add_type (AnjutaMessageManager * amm, gint type_name,
 		}
 	}
 
-	MessageSubwindow *sub_win;
-	if (type_name != MESSAGE_TERMINAL)
-	{
-		AnjutaMessageWindow *window =
-			new AnjutaMessageWindow (amm, type_name, type, pixmap);
-		gtk_signal_connect (GTK_OBJECT (window->get_msg_list ()),
-				    "select_row",
-				    GTK_SIGNAL_FUNC (on_msg_double_clicked),
-				    window);
-		sub_win = window;
-	}
-	else
-	{
-		TerminalWindow *window =
-			new TerminalWindow (amm, type_name, type, pixmap);
-		sub_win = window;
-	}
+	MessageSubwindow *sub_win;	
+	switch(type_name) {
+		case MESSAGE_TERMINAL:
+			sub_win = new TerminalWindow (amm, type_name, type, pixmap);
+			break;
+		case MESSAGE_LOCALS:
+		    sub_win = new LocalsWindow(amm, type_name, type, pixmap);
+			break;
+		case MESSAGE_WATCHES:
+		case MESSAGE_STACK:
+			sub_win = new WidgetWindow(amm, type_name, type, pixmap);
+			break;
+		default:
+			sub_win = new AnMessageWindow (amm, type_name, type, pixmap);
+			g_signal_connect (GTK_OBJECT (
+				dynamic_cast<AnMessageWindow*>(sub_win)->get_msg_list()),
+				"button_press_event", GTK_SIGNAL_FUNC (on_popup_clicked), 
+				sub_win->get_parent()->intern);
+		}
 	amm->intern->msg_windows.push_back (sub_win);
 	if (amm->intern->last_page == 0)
 		amm->intern->msg_windows.back ()->activate ();
@@ -233,14 +234,18 @@ anjuta_message_manager_add_type (AnjutaMessageManager * amm, gint type_name,
 
 // Adds a string to a message_window and shows it when a newline appears
 gboolean
-anjuta_message_manager_append (AnjutaMessageManager * amm,
+an_message_manager_append (AnMessageManager * amm,
 			       const gchar * msg_string, gint type_name)
 {
-	if (!amm)
-		return false;
+	g_return_val_if_fail (amm != NULL, false);
 	
 	string type = labels[type_name];
-	string msg = msg_string;
+	
+	gchar *utf8_str = anjuta_util_convert_to_utf8 (msg_string);
+	g_return_val_if_fail (utf8_str != NULL, false);
+	string msg = utf8_str;
+	g_free (utf8_str);
+	
 	vector < MessageSubwindow * >::iterator cur_win;
 
 	if (!amm->intern->msg_windows.empty ())
@@ -252,7 +257,7 @@ anjuta_message_manager_append (AnjutaMessageManager * amm,
 			if ((*cur_win)->get_type () == type)
 			{
 				found = true;
-				if (!dynamic_cast < AnjutaMessageWindow * >(*cur_win))
+				if (!dynamic_cast < AnMessageWindow * >(*cur_win))
 					return false;
 				if (!(*cur_win)->is_shown ())
 				{
@@ -272,8 +277,8 @@ anjuta_message_manager_append (AnjutaMessageManager * amm,
 	{
 		return false;
 	}
-	AnjutaMessageWindow *window =
-		dynamic_cast < AnjutaMessageWindow * >(*cur_win);
+	AnMessageWindow *window =
+		dynamic_cast < AnMessageWindow * >(*cur_win);
 	for (string::iterator c = msg.begin (); c != msg.end (); c++)
 	{
 		if (*c == '\n')
@@ -288,91 +293,189 @@ anjuta_message_manager_append (AnjutaMessageManager * amm,
 	return true;
 }
 
+gboolean
+an_message_manager_show_pane (AnMessageManager* amm, gint type_name)
+{
+	g_return_val_if_fail (amm != NULL, false);
+	
+	string type = labels[type_name];
+	
+	vector < MessageSubwindow * >::iterator cur_win;
+
+	if (!amm->intern->msg_windows.empty ())
+	{
+		bool found = false;
+		for (cur_win = amm->intern->msg_windows.begin ();
+		     cur_win != amm->intern->msg_windows.end (); cur_win++)
+		{
+			if ((*cur_win)->get_type () == type)
+			{
+				found = true;
+				if (!dynamic_cast < AnMessageWindow * >(*cur_win))
+					return false;
+				if (!(*cur_win)->is_shown ())
+				{
+					(*cur_win)->activate ();
+				}
+				break;
+			}
+		}
+		if (!found)
+		{
+			g_warning (_("Could not find message type %s!\n"),
+				   type.c_str ());
+			return false;
+		}
+	}
+	return true;
+}
+
+// Returns the subwindow according to the requested type
+MessageSubwindow *
+an_message_manager_get_window(AnMessageManager * amm,gint type_name)
+{
+	if (!amm)
+	  return NULL;
+	
+	string type = labels[type_name];
+	vector < MessageSubwindow * >::iterator cur_win;
+
+	if (!amm->intern->msg_windows.empty ())
+	{
+		bool found = false;
+		for (cur_win = amm->intern->msg_windows.begin ();
+		     cur_win != amm->intern->msg_windows.end (); cur_win++)
+		{
+			if ((*cur_win)->get_type () == type)
+			{
+				found = true;
+				//				if (!dynamic_cast < AnMessageWindow * >(*cur_win))
+				//return NULL;
+				if (!(*cur_win)->is_shown ())
+				{
+					(*cur_win)->activate ();
+				}
+				break;
+			}
+		}
+		if (!found)
+		{
+			g_warning (_("Could not find message type %s!\n"),
+				   type.c_str ());
+			return NULL;
+		}
+	}
+	else
+	{
+		return NULL;
+	}
+	return *cur_win;
+}
+
 // Sent a "message_click" event as if the next message was clicked
 void
-anjuta_message_manager_next (AnjutaMessageManager * amm)
+an_message_manager_next (AnMessageManager * amm)
 {
 	g_return_if_fail(amm != NULL);
-	AnjutaMessageWindow *win =
+	AnMessageWindow *win =
 		dynamic_cast <
-		AnjutaMessageWindow * >(amm->intern->cur_msg_win);
+		AnMessageWindow * >(amm->intern->cur_msg_win);
 	g_return_if_fail (win != 0);
 	// Fix for bug #509192 (Crash on next message)
 	if (win->get_messages().empty())
 		return;
-	if (win->get_cur_line () < (win->get_messages ().size () - 1))
+	
+	gchar* file;
+	int line;
+	win->select_next();
+	while (!parse_error_line(win->get_cur_msg().c_str(), &file, &line))
 	{
-		gchar* file;
-		int line;
-		if (!parse_error_line(win->get_messages()[win->get_cur_line() + 1].c_str(), &file, &line))
-		{
-			gtk_clist_select_row (GTK_CLIST (win->get_msg_list ()),
-				      win->get_cur_line () + 1, 0);
-			anjuta_message_manager_next(amm);
+		if (!win->select_next())			
 			return;
-		}
-		else
-			delete[] file;		
-		
-		gtk_signal_emit (GTK_OBJECT (amm),
-				 anjuta_message_manager_signals
-				 [MESSAGE_CLICKED],
-				 win->get_messages ()[win->get_cur_line() + 1].c_str ());
-		// this increments win->get_cur_line, so + 1 is not needed after it
-		gtk_clist_select_row (GTK_CLIST (win->get_msg_list ()),
-				      win->get_cur_line () + 1, 0);
-		
-		GtkAdjustment* adj = gtk_clist_get_vadjustment (GTK_CLIST(win->get_msg_list ()));
-		float value = (adj->upper*win->get_cur_line())/win->get_messages().size() - adj->page_increment/2;
-		value = MAX (0, value);
-		value = MIN (value, adj->upper - adj->page_increment);
-		gtk_adjustment_set_value (adj, value);
+		delete[] file;		
 	}
+	on_message_clicked (amm,
+			 win->get_cur_msg().c_str());
+	
+	GtkAdjustment* adj = gtk_scrolled_window_get_vadjustment (
+		GTK_SCROLLED_WINDOW(win->get_scrolled_win ()));
+	float value = (adj->upper*win->get_cur_line())/win->get_messages().size() - adj->page_increment/2;
+	value = MAX (0, value);
+	value = MIN (value, adj->upper - adj->page_increment);
+	gtk_adjustment_set_value (adj, value);
 }
 
 // Sent a "message_click" event as if the previous message was clicked
 void
-anjuta_message_manager_previous (AnjutaMessageManager * amm)
+an_message_manager_previous (AnMessageManager * amm)
 {
 	g_return_if_fail(amm != NULL);
-	AnjutaMessageWindow *win =
-		dynamic_cast < AnjutaMessageWindow * >(amm->intern->cur_msg_win);
+	AnMessageWindow *win =
+		dynamic_cast < AnMessageWindow * >(amm->intern->cur_msg_win);
 	g_return_if_fail (win != 0);
 	// Fix for bug #509192 (Crash on next message)
 	if (win->get_messages().empty())
 		return;
-	if (win->get_cur_line () > 0)
+	gchar* file;
+	int line;
+	win->select_prev();
+	while (!parse_error_line(win->get_cur_msg().c_str(), &file, &line))
 	{
-		gchar* file;
-		int line;
-		if (!parse_error_line(win->get_messages()[win->get_cur_line() - 1].c_str(), &file, &line))
-		{
-			gtk_clist_select_row (GTK_CLIST (win->get_msg_list ()),
-				      win->get_cur_line () - 1, 0);
-			anjuta_message_manager_previous(amm);
-			return;
-		}
-		else
-			delete[] file;
-		
-		gtk_signal_emit (GTK_OBJECT (amm),
-				 anjuta_message_manager_signals
-				 [MESSAGE_CLICKED],
-				 win->get_messages ()[win->get_cur_line () - 1].c_str ());
-		// see above...
-		gtk_clist_select_row (GTK_CLIST (win->get_msg_list ()),
-				      win->get_cur_line () - 1, 0);
-		
-		GtkAdjustment* adj = gtk_clist_get_vadjustment (GTK_CLIST(win->get_msg_list ()));
-		float value = (adj->upper*win->get_cur_line())/win->get_messages().size() - adj->page_increment/2;
-		value = MAX (0, value);
-		value = MIN (value, adj->upper - adj->page_increment);
-		gtk_adjustment_set_value (adj, value);
+		if (!win->select_prev())
+			return;		
+		delete[] file;
 	}
+		
+	on_message_clicked (amm,
+			 win->get_cur_msg().c_str());
+	GtkAdjustment* adj = gtk_scrolled_window_get_vadjustment (
+		GTK_SCROLLED_WINDOW(win->get_scrolled_win ()));
+	float value = (adj->upper*win->get_cur_line())/win->get_messages().size() - adj->page_increment/2;
+	value = MAX (0, value);
+	value = MIN (value, adj->upper - adj->page_increment);
+	gtk_adjustment_set_value (adj, value);
 }
 
 void
-anjuta_message_manager_show (AnjutaMessageManager * amm, gint type_name)
+an_message_manager_indicate_error (AnMessageManager * amm, gint type_name,
+		gchar* file, glong line)
+{
+	static MessageIndicatorInfo info;
+	if (type_name) return; // Only for Build messages.
+	info.type = type_name;
+	info.filename = file;
+	info.line = line;
+	info.message_type = MESSAGE_INDICATOR_ERROR;
+	on_message_indicate(amm, &info);
+}
+
+void
+an_message_manager_indicate_warning (AnMessageManager * amm, gint type_name,
+		gchar* file, glong line)
+{
+	static MessageIndicatorInfo info;
+	if (type_name) return; // Only for Build messages.
+	info.type = type_name;
+	info.filename = file;
+	info.line = line;
+	info.message_type = MESSAGE_INDICATOR_WARNING;
+	on_message_indicate(amm, &info);
+}
+
+void
+an_message_manager_indicate_others (AnMessageManager * amm, gint type_name,
+		gchar* file, glong line)
+{
+	static MessageIndicatorInfo info;
+	info.type = type_name;
+	info.filename = file;
+	info.line = line;
+	info.message_type = MESSAGE_INDICATOR_OTHERS;
+	on_message_indicate(amm, &info);
+}
+
+void
+an_message_manager_show (AnMessageManager * amm, gint type_name)
 {
 	g_return_if_fail(amm != NULL);
 	if (type_name != MESSAGE_NONE)
@@ -389,7 +492,7 @@ anjuta_message_manager_show (AnjutaMessageManager * amm, gint type_name)
 			}
 		}
 	}
-	if (anjuta_message_manager_is_shown (amm))
+	if (an_message_manager_is_shown (amm))
 	{
 		if (!amm->intern->is_docked && amm->intern->window)
 			gdk_window_raise(amm->intern->window->window);
@@ -405,30 +508,36 @@ anjuta_message_manager_show (AnjutaMessageManager * amm, gint type_name)
 		if (amm->intern->window == 0)
 		{
 			amm->intern->is_docked = true;
-			anjuta_message_manager_undock (amm);
+			an_message_manager_undock (amm);
 		}
 		gtk_widget_show_all(amm->intern->window);
 	}
 }
 
 void
-anjuta_message_manager_info_locals (AnjutaMessageManager * amm, GList * lines,
+an_message_manager_info_locals (AnMessageManager * amm, GList * lines,
 				    gpointer data)
 {
 	g_return_if_fail (amm != NULL);
 	if (g_list_length (lines) < 1)
 		return;
-	while (lines)
+	MessageSubwindow* window = an_message_manager_get_window(amm,MESSAGE_LOCALS);
+	if (!window) 
 	{
-		anjuta_message_manager_append (amm, (char *) lines->data,
-					       MESSAGE_LOCALS);
-		anjuta_message_manager_append (amm, "\n", MESSAGE_LOCALS);
-		lines = g_list_next (lines);
+		// g_print ("an_message_manager_get_window error\n");
+		return;
 	}
+	LocalsWindow* locals_window = dynamic_cast<LocalsWindow*>(window);
+	if (!locals_window)
+	{
+		// g_print ("dynamic cast to locals window error\n");
+		return;
+	}
+	locals_window->update_view(lines);
 }
 
 void
-anjuta_message_manager_clear (AnjutaMessageManager * amm, gint type_name)
+an_message_manager_clear (AnMessageManager * amm, gint type_name)
 {
 	g_return_if_fail(amm != NULL);
 	
@@ -438,16 +547,17 @@ anjuta_message_manager_clear (AnjutaMessageManager * amm, gint type_name)
 	{
 		if ((*cur_win)->get_type () == string (labels[type_name]))
 		{
-			AnjutaMessageWindow *win = dynamic_cast < AnjutaMessageWindow * >(*cur_win);
-			if (win != 0)
-				win->clear ();
+			(*cur_win)->clear();
+			if (type_name == 0) {
+				anjuta_delete_all_indicators();
+			}
 			break;
 		}
 	}
 }
 
 void
-anjuta_message_manager_dock (AnjutaMessageManager * amm)
+an_message_manager_dock (AnMessageManager * amm)
 {
 	g_return_if_fail(amm != NULL);
 	
@@ -465,7 +575,7 @@ anjuta_message_manager_dock (AnjutaMessageManager * amm)
 }
 
 void
-anjuta_message_manager_undock (AnjutaMessageManager * amm)
+an_message_manager_undock (AnMessageManager * amm)
 {
 	g_return_if_fail(amm != NULL);
 	
@@ -474,11 +584,17 @@ anjuta_message_manager_undock (AnjutaMessageManager * amm)
 		amm->intern->is_docked = false;
 		
 		amm->intern->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-		gtk_window_set_wmclass(GTK_WINDOW(amm->intern->window), "message-manager", "anjuta");
+		/*
+		gtk_window_set_transient_for(GTK_WINDOW(amm->intern->window), GTK_WINDOW(app->widgets.window));
+		*/
+		gnome_window_icon_set_from_default((GtkWindow *) amm->intern->window);
+		gtk_window_set_wmclass(GTK_WINDOW(amm->intern->window),
+							   "message-manager", "an");
 		gtk_window_set_title(GTK_WINDOW(amm->intern->window), _("Messages"));
-		gtk_window_set_default_size(GTK_WINDOW(amm->intern->window), amm->intern->width, amm->intern->height);
-		gtk_widget_set_uposition(amm->intern->window, amm->intern->xpos, amm->intern->ypos);
-		
+		gtk_window_set_default_size(GTK_WINDOW(amm->intern->window),
+									amm->intern->width, amm->intern->height);
+		gtk_window_move (GTK_WINDOW (amm->intern->window),
+						 amm->intern->xpos, amm->intern->ypos);
 		amm_undock (GTK_WIDGET (amm), &amm->intern->window);
 		if (amm->intern->is_shown)
 			gtk_widget_show (GTK_WIDGET (amm->intern->window));
@@ -486,7 +602,7 @@ anjuta_message_manager_undock (AnjutaMessageManager * amm)
 }
 
 gboolean
-anjuta_message_manager_is_shown (AnjutaMessageManager * amm)
+an_message_manager_is_shown (AnMessageManager * amm)
 {
 	if (!amm)
 		return false;
@@ -494,19 +610,20 @@ anjuta_message_manager_is_shown (AnjutaMessageManager * amm)
 }
 
 gboolean
-anjuta_message_manager_save_yourself (AnjutaMessageManager * amm,
+an_message_manager_save_yourself (AnMessageManager * amm,
 				      FILE * stream)
 {
 	if (!amm)
 		return false;
-	
+
+	fprintf (stream, "messages.is.shown=%d\n", amm->intern->is_shown);
 	fprintf (stream, "messages.is.docked=%d\n", amm->intern->is_docked);
 	if (amm->intern->is_shown && !amm->intern->is_docked)
 	{
 		gdk_window_get_root_origin (amm->intern->window->window,
 					    &amm->intern->xpos,
 					    &amm->intern->ypos);
-		gdk_window_get_size (amm->intern->window->window,
+		gtk_window_get_size (GTK_WINDOW(amm->intern->window),
 				     &amm->intern->width,
 				     &amm->intern->height);
 	}
@@ -528,21 +645,22 @@ anjuta_message_manager_save_yourself (AnjutaMessageManager * amm,
 }
 
 gboolean
-anjuta_message_manager_load_yourself (AnjutaMessageManager * amm,
+an_message_manager_load_yourself (AnMessageManager * amm,
 				      PropsID props)
 {
 	if (!amm)
 		return false;
+	bool shown_flag;
 	bool dock_flag;
 	if (!amm)
 		return false;
 
-	dock_flag = prop_get_int (props, "messages.is.docked", 0);
+	shown_flag = prop_get_int (props, "messages.is.shown", 1);
+	dock_flag = prop_get_int (props, "messages.is.docked", 1);
 	amm->intern->xpos = prop_get_int (props, "messages.win.pos.x", 50);
 	amm->intern->ypos = prop_get_int (props, "messages.win.pos.y", 50);
 	amm->intern->width = prop_get_int (props, "messages.win.width", 600);
-	amm->intern->height =
-		prop_get_int (props, "messages.win.height", 300);
+	amm->intern->height = prop_get_int (props, "messages.win.height", 300);
 	
 	typedef vector < MessageSubwindow * >::iterator I;
 	for (I cur_win = amm->intern->msg_windows.begin ();
@@ -559,88 +677,98 @@ anjuta_message_manager_load_yourself (AnjutaMessageManager * amm,
 		delete[] str;
 	}
 	
-	anjuta_message_manager_update(amm);
-	
+	an_message_manager_update(amm);
+
 	amm_hide_docked ();
 	if (!dock_flag)
-		anjuta_message_manager_undock (amm);
+		an_message_manager_undock (amm);
 	else
 		amm->intern->is_docked = true;
+	if (shown_flag)
+	{
+		an_message_manager_show (amm, MESSAGE_NONE);
+	}
 	return true;
 }
 
 void
-anjuta_message_manager_update(AnjutaMessageManager* amm)
+an_message_manager_update(AnMessageManager* amm)
 {
 	g_return_if_fail(amm != NULL);
 	
-	Preferences* p = get_preferences();
+	AnjutaPreferences* p = get_preferences();
 	
-	char* tag_pos = preferences_get(p, MESSAGES_TAG_POS);
+	char* tag_pos = anjuta_preferences_get(p, MESSAGES_TAG_POS);
 	if (tag_pos != NULL)
 	{
-		if (strcmp(tag_pos, "top")==0)
+		if (strcasecmp(tag_pos, "top")==0)
 			gtk_notebook_set_tab_pos(GTK_NOTEBOOK(amm->intern->notebook), GTK_POS_TOP);
-		else if (strcmp(tag_pos, "left")==0)
+		else if (strcasecmp(tag_pos, "left")==0)
 			gtk_notebook_set_tab_pos(GTK_NOTEBOOK(amm->intern->notebook), GTK_POS_LEFT);
-		if (strcmp(tag_pos, "bottom")==0)
-			gtk_notebook_set_tab_pos(GTK_NOTEBOOK(amm->intern->notebook), GTK_POS_BOTTOM);
-		if (strcmp(tag_pos, "right")==0)
+		else if (strcasecmp(tag_pos, "right")==0)
 			gtk_notebook_set_tab_pos(GTK_NOTEBOOK(amm->intern->notebook), GTK_POS_RIGHT);
+		else
+			gtk_notebook_set_tab_pos(GTK_NOTEBOOK(amm->intern->notebook), GTK_POS_BOTTOM);
+		
 		g_free(tag_pos);
 	}
 	
 	guint8 r, g, b;
 	guint factor = ((guint16) -1) / ((guint8) -1);
 	char* color;
-	color = preferences_get(p, MESSAGES_COLOR_ERROR);
+	color = anjuta_preferences_get(p, MESSAGES_COLOR_ERROR);
 	if (color)
 	{
-		ColorFromString(color, &r, &g, &b);
+		anjuta_util_color_from_string (color, &r, &g, &b);
+		amm->intern->color_error.pixel = 0;
 		amm->intern->color_error.red = r * factor;
 		amm->intern->color_error.green = g * factor;
 		amm->intern->color_error.blue = b * factor;
 		g_free(color);
 	}
 	
-	color = preferences_get(p, MESSAGES_COLOR_WARNING);
+	color = anjuta_preferences_get (p, MESSAGES_COLOR_WARNING);
 	if (color)
 	{
-		ColorFromString(color, &r, &g, &b);
+		anjuta_util_color_from_string (color, &r, &g, &b);
+		amm->intern->color_warning.pixel = 0;
 		amm->intern->color_warning.red = r * factor;
 		amm->intern->color_warning.green = g * factor;
 		amm->intern->color_warning.blue = b * factor;
 		g_free(color);
 	}
-	color = preferences_get(p, MESSAGES_COLOR_MESSAGES1);
+	color = anjuta_preferences_get(p, MESSAGES_COLOR_MESSAGES1);
 	if (color)
 	{
-		ColorFromString(color, &r, &g, &b);
+		anjuta_util_color_from_string (color, &r, &g, &b);
+		amm->intern->color_message1.pixel = 0;
 		amm->intern->color_message1.red = r * factor;
 		amm->intern->color_message1.green = g * factor;
 		amm->intern->color_message1.blue = b * factor;
 		g_free(color);
 	}
-	color = preferences_get(p, MESSAGES_COLOR_MESSAGES2);
+	color = anjuta_preferences_get(p, MESSAGES_COLOR_MESSAGES2);
 	if (color)
 	{
-		ColorFromString(color, &r, &g, &b);
+		anjuta_util_color_from_string (color, &r, &g, &b);
+		amm->intern->color_message2.pixel = 0;
 		amm->intern->color_message2.red = r * factor;
 		amm->intern->color_message2.green = g * factor;
 		amm->intern->color_message2.blue = b * factor;
 		g_free(color);
 	}
+
+	anjuta_delete_all_indicators();
 	
 	typedef vector < MessageSubwindow * >::iterator I;
 	for (I cur_win = amm->intern->msg_windows.begin ();
 	     cur_win != amm->intern->msg_windows.end (); cur_win++)
 	{
-		AnjutaMessageWindow* win = dynamic_cast<AnjutaMessageWindow*>(*cur_win);
+		AnMessageWindow* win = dynamic_cast<AnMessageWindow*>(*cur_win);
 		if (!win)
 			break;
 		
 		vector<string> messages = win->get_messages();
-		win->freeze();
 		win->clear();
 		int type = win->get_type_id();
 		
@@ -648,20 +776,19 @@ anjuta_message_manager_update(AnjutaMessageManager* amm)
 		for (CI cur_msg = messages.begin(); cur_msg != messages.end(); cur_msg++)
 		{
 			string msg = *cur_msg + "\n";
-			anjuta_message_manager_append(amm, msg.c_str(), type);
+			an_message_manager_append(amm, msg.c_str(), type);
 		}
-		win->thaw();
 	}
 }
 
 gboolean
-anjuta_message_manager_save_build (AnjutaMessageManager* amm, FILE * stream)
+an_message_manager_save_build (AnMessageManager* amm, FILE * stream)
 {
 	typedef vector < MessageSubwindow * >::iterator I;
 	for (I cur_win = amm->intern->msg_windows.begin ();
 	     cur_win != amm->intern->msg_windows.end (); cur_win++)
 	{
-		AnjutaMessageWindow* win = dynamic_cast<AnjutaMessageWindow*>(*cur_win);
+		AnMessageWindow* win = dynamic_cast<AnMessageWindow*>(*cur_win);
 		if (!win)
 			break;
 		if ((*cur_win)->get_type_id() == MESSAGE_BUILD)
@@ -679,13 +806,13 @@ anjuta_message_manager_save_build (AnjutaMessageManager* amm, FILE * stream)
 }
 
 gboolean
-anjuta_message_manager_build_is_empty(AnjutaMessageManager* amm)
+an_message_manager_build_is_empty(AnMessageManager* amm)
 {
 	typedef vector < MessageSubwindow * >::iterator I;
 	for (I cur_win = amm->intern->msg_windows.begin ();
 	     cur_win != amm->intern->msg_windows.end (); cur_win++)
 	{
-		AnjutaMessageWindow* win = dynamic_cast<AnjutaMessageWindow*>(*cur_win);
+		AnMessageWindow* win = dynamic_cast<AnMessageWindow*>(*cur_win);
 		if (!win)
 			break;
 		if ((*cur_win)->get_type_id() == MESSAGE_BUILD)
@@ -696,21 +823,48 @@ anjuta_message_manager_build_is_empty(AnjutaMessageManager* amm)
 	return true;
 }
 		
+
+void
+an_message_manager_set_widget(AnMessageManager* amm, gint type_name,
+							  GtkWidget* widget)
+{
+	MessageSubwindow* msb = NULL;
+	WidgetWindow* ww = NULL;
+
+	msb = an_message_manager_get_window(amm, type_name);
+	
+	if (!msb) 
+	{
+		g_warning("No window type: %d\n",type_name);
+		return;
+	}
+	
+	ww = dynamic_cast<WidgetWindow*>(msb);
+	
+	if (!ww)
+	{
+		g_warning("Cannot case to widget window");
+		return;
+	}
+	
+	ww->set_widget(widget);
+}
+
 // Private:
 
 // Callbacks:
 
 static void
-anjuta_message_manager_show_impl (GtkWidget * widget)
+an_message_manager_show_impl (GtkWidget * widget)
 {
-	AnjutaMessageManager *amm = ANJUTA_MESSAGE_MANAGER (widget);
-	g_return_if_fail  (ANJUTA_IS_MESSAGE_MANAGER(amm));
+	AnMessageManager *amm = AN_MESSAGE_MANAGER (widget);
+	g_return_if_fail  (AN_IS_MESSAGE_MANAGER(amm));
 	
 	if (!amm->intern->is_docked)
 	{
-		gtk_widget_set_uposition (amm->intern->window,
-					  amm->intern->xpos,
-					  amm->intern->ypos);
+		gtk_window_move (GTK_WINDOW (amm->intern->window),
+						 amm->intern->xpos,
+						 amm->intern->ypos);
 		gtk_window_set_default_size (GTK_WINDOW (amm->intern->window),
 					     amm->intern->width,
 					     amm->intern->height);
@@ -720,9 +874,9 @@ anjuta_message_manager_show_impl (GtkWidget * widget)
 }
 
 static void
-anjuta_message_manager_hide_impl (GtkWidget * widget)
+an_message_manager_hide_impl (GtkWidget * widget)
 {
-	AnjutaMessageManager *amm = ANJUTA_MESSAGE_MANAGER (widget);
+	AnMessageManager *amm = AN_MESSAGE_MANAGER (widget);
 	if (amm->intern->is_docked)
 	{
 		amm_hide_docked ();
@@ -732,7 +886,7 @@ anjuta_message_manager_hide_impl (GtkWidget * widget)
 		gdk_window_get_root_origin (amm->intern->window->window,
 					    &amm->intern->xpos,
 					    &amm->intern->ypos);
-		gdk_window_get_size (amm->intern->window->window,
+		gtk_window_get_size (GTK_WINDOW(amm->intern->window),
 				     &amm->intern->width,
 				     &amm->intern->height);
 		gtk_widget_hide (amm->intern->window);
@@ -741,38 +895,40 @@ anjuta_message_manager_hide_impl (GtkWidget * widget)
 }
 
 static gboolean
-on_dock_activate (GtkWidget * menuitem, AnjutaMessageManager * amm)
+on_dock_activate (GtkWidget * menuitem, AnMessageManager * amm)
 {
 	if (!amm)
 		return false;
 	
 	if (amm->intern->is_docked)
 	{
-		anjuta_message_manager_undock (amm);
+		an_message_manager_undock (amm);
 	}
 	else
 	{
-		anjuta_message_manager_dock (amm);
+		an_message_manager_dock (amm);
 	}
 	return true;
 }
 
 static gboolean
-on_popup_clicked (AnjutaMessageManager * amm, GdkEvent * event)
+on_popup_clicked (GtkWidget* widget, GdkEvent * event, 
+	AnMessageManagerPrivate* intern)
 {
-	if (!amm)
-		return false;
+	if (!intern || !event)
+		return FALSE;
 	
 	if (event->type == GDK_BUTTON_PRESS
 	    && ((GdkEventButton *) event)->button == 3)
 	{
-		gtk_menu_popup (GTK_MENU (amm->intern->popupmenu), NULL, NULL,
+		GTK_CHECK_MENU_ITEM(intern->dock_item)->active = intern->is_docked;
+		gtk_menu_popup (GTK_MENU (intern->popupmenu), NULL, NULL,
 				NULL, NULL,
 				((GdkEventButton *) event)->button,
 				((GdkEventButton *) event)->time);
-		return true;
+		return TRUE;
 	}
-	return false;
+	return FALSE;
 }
 
 static gboolean
@@ -780,7 +936,7 @@ on_show_hide_tab (GtkWidget * menuitem, MessageSubwindow * msg_win)
 {
 	if (menuitem == NULL)
 		return false;
-	AnjutaMessageManager *amm = msg_win->get_parent ();
+	AnMessageManager *amm = msg_win->get_parent ();
 	if (msg_win->is_shown ())
 	{
 		msg_win->hide ();
@@ -807,67 +963,103 @@ on_show_hide_tab (GtkWidget * menuitem, MessageSubwindow * msg_win)
 	return true;
 }
 
-// Called when a message in the list is double clicked. Emits a the "message_clicked" signal
-void
-on_msg_double_clicked (GtkCList * list, gint row, gint column,
-		       GdkEvent * event, gpointer data)
-{
-	g_return_if_fail(data != NULL);
-	AnjutaMessageWindow *win =
-			reinterpret_cast < AnjutaMessageWindow * >(data);
-	win->set_cur_line (row);
-	if (event == NULL)
-		return;
-	if (event->type == GDK_BUTTON_PRESS
-	    && ((GdkEventButton *) event)->button == 1)
-	{
-		return;
-	}
-	if (event->type != GDK_2BUTTON_PRESS)
-		return;
-	if (((GdkEventButton *) event)->button != 1)
-		return;
-
-	gtk_signal_emit (GTK_OBJECT (win->get_parent ()),
-			 anjuta_message_manager_signals[MESSAGE_CLICKED], win->get_messages()[row].c_str());
-	return;
-}
-
 // Utilities:
 
 void
-create_default_types (AnjutaMessageManager * amm)
+create_default_types (AnMessageManager * amm)
 {
 	g_return_if_fail(amm != NULL);
-	anjuta_message_manager_add_type (amm, MESSAGE_BUILD,
+	an_message_manager_add_type (amm, MESSAGE_BUILD,
 					 ANJUTA_PIXMAP_MINI_BUILD);
-	anjuta_message_manager_add_type (amm, MESSAGE_DEBUG,
-					 ANJUTA_PIXMAP_MINI_DEBUG);
-	anjuta_message_manager_add_type (amm, MESSAGE_FIND,
+	an_message_manager_add_type (amm, MESSAGE_FIND,
 					 ANJUTA_PIXMAP_MINI_FIND);
-	anjuta_message_manager_add_type (amm, MESSAGE_CVS,
+	an_message_manager_add_type (amm, MESSAGE_CVS,
 					 ANJUTA_PIXMAP_MINI_CVS);
-	anjuta_message_manager_add_type (amm, MESSAGE_LOCALS,
-					 ANJUTA_PIXMAP_MINI_LOCALS);
-	anjuta_message_manager_add_type (amm, MESSAGE_TERMINAL,
+	an_message_manager_add_type (amm, MESSAGE_TERMINAL,
 					 ANJUTA_PIXMAP_MINI_TERMINAL);
+	an_message_manager_add_type (amm, MESSAGE_STDOUT,
+					 ANJUTA_PIXMAP_MINI_TERMINAL);
+	an_message_manager_add_type (amm, MESSAGE_STDERR,
+					 ANJUTA_PIXMAP_MINI_TERMINAL);
+	an_message_manager_add_type (amm, MESSAGE_DEBUG,
+					 ANJUTA_PIXMAP_MINI_DEBUG);
+	an_message_manager_add_type (amm, MESSAGE_LOCALS,
+					 ANJUTA_PIXMAP_MINI_LOCALS);
+	an_message_manager_add_type (amm, MESSAGE_WATCHES,
+					 ANJUTA_PIXMAP_MINI_LOCALS);	
+	an_message_manager_add_type (amm, MESSAGE_STACK,
+					 ANJUTA_PIXMAP_MINI_LOCALS);	
 	
 	// Fix for bug #509192 (Crash on next message)
 	amm->intern->cur_msg_win = 
-			dynamic_cast<AnjutaMessageWindow*>(*(amm->intern->msg_windows.begin()));
+			dynamic_cast<AnMessageWindow*>(*(amm->intern->msg_windows.begin()));
 }
 
 void
 connect_menuitem_signal (GtkWidget * item, MessageSubwindow * msg_win)
 {
-	gtk_signal_connect (GTK_OBJECT (item), "toggled",
-			    GTK_SIGNAL_FUNC (on_show_hide_tab), msg_win);
+	g_signal_connect (G_OBJECT (item), "toggled",
+			    G_CALLBACK (on_show_hide_tab), msg_win);
 }
 
 void
 disconnect_menuitem_signal (GtkWidget * item, MessageSubwindow * msg_win)
 {
-	gtk_signal_disconnect_by_func (GTK_OBJECT (item),
-				       GTK_SIGNAL_FUNC (on_show_hide_tab),
-				       msg_win);
+	// g_signal_stop_emission_by_name (G_OBJECT (item), "toggled");
+	g_signal_handlers_disconnect_by_func (G_OBJECT (item),
+										  (void *) on_show_hide_tab,
+										  msg_win);
+}
+
+void
+on_message_clicked(AnMessageManager* amm, const char* message)
+{
+	char* fn;
+	int ln;
+	if (parse_error_line (message, &fn, &ln))
+	{
+		anjuta_goto_file_line (fn, ln);
+		g_free (fn);
+	}
+}
+
+void
+on_message_indicate (AnMessageManager* amm, MessageIndicatorInfo *info)
+{
+	gchar *fn;
+	GList *node;
+
+	TextEditor *te;
+	if (!anjuta_preferences_get_int_with_default(ANJUTA_PREFERENCES
+												 (app->preferences),
+											MESSAGES_INDICATORS_AUTOMATIC, 1)) {
+		return;
+	}
+	g_return_if_fail (info);
+	/* Is always NULL if no error or warning is indicated!
+	g_return_if_fail (info->filename); */
+	if (info->filename == NULL)
+		return;
+	fn = anjuta_get_full_filename (info->filename);
+	g_return_if_fail (fn);
+	
+	node = app->text_editor_list;
+	while (node)
+	{
+		te = (TextEditor *) node->data;
+		if (te->full_filename == NULL)
+		{
+			node = g_list_next (node);
+			continue;
+		}
+		if (strcmp (fn, te->full_filename) == 0)
+		{
+			if (info->line >= 0)
+				text_editor_set_indicator (te, info->line, info->message_type);
+			g_free (fn);
+			return;
+		}
+		node = g_list_next (node);
+	}
+	g_free (fn);
 }
