@@ -22,9 +22,9 @@
 
 #include <glib.h>
 
-typedef struct _Project_Type Project_Type;
+typedef struct _ProjectType ProjectType;
 
-struct _Project_Type
+struct _ProjectType
 {
 	gint id;
 	gchar* name;
@@ -38,13 +38,12 @@ struct _Project_Type
 	gchar* autogen_file;
 	
 	gboolean gnome_support;
+	gboolean gnome2_support;
 	gboolean glade_support;
+	gboolean deprecated;
 };
 
-Project_Type* 
-load_project_type(gint ID);
-
-void 
-free_project_type(Project_Type *type);
+ProjectType* project_type_load (gint ID);
+void project_type_free (ProjectType *type);
 
 #endif
