@@ -96,7 +96,7 @@ get_propset(PropsID pi)
   PropSetFile* p;
   if(pi < 0 || (guint)pi >= g_list_length(anjuta_propset))
   {
-  	g_warning("Invalind PropSetFile handle");
+  	g_warning("Invalid PropSetFile handle");
   	return NULL;
   }
   p = (PropSetFile*)g_list_nth_data(anjuta_propset, pi);
@@ -165,17 +165,26 @@ prop_set_with_key(PropsID handle, const gchar *key, const gchar *val)
 	  p->Set(key, "");
 }
 
+void 
+prop_set_int_with_key (PropsID p, const gchar *key, int value)
+{
+  gchar *str;
+  str = g_strdup_printf ("%d", value);
+  prop_set_with_key (p, key, str);
+  g_free (str);  
+}
+
 void
-prop_set(PropsID handle, gchar *keyval)
+prop_set(PropsID handle, const gchar *keyval)
 {
   PropSetFile* p;
-  p = get_propset(handle);
+  p = get_propset (handle);
   if(!p) return;
   p->Set(keyval);
 }
 
 gchar*
-prop_get(PropsID handle, gchar *key)
+prop_get(PropsID handle, const gchar *key)
 {
   PropSetFile* p;
   SString s;
