@@ -19,9 +19,19 @@
 #ifndef _RESOURCES_H_
 #define _RESOURCES_H_
 
-GtkWidget* anjuta_res_lookup_widget (GtkWidget       *widget, const gchar     *widget_name);
-GtkWidget* anjuta_res_get_pixmap_widget (GtkWidget* widget, const gchar* pixfile, gboolean gnome_pixmap);
-GdkImlibImage* anjuta_res_get_image  (const gchar     *filename);
+#ifdef __cplusplus
+extern "C"
+{
+#endif
+
+#include "properties.h"
+
+GtkWidget* anjuta_res_lookup_widget (GtkWidget *widget,
+									 const gchar *widget_name);
+GtkWidget* anjuta_res_get_image (const gchar* filename);
+	
+/* Do not unref the returned value  */
+GdkPixbuf* anjuta_res_get_pixbuf  (const gchar *filename);
 
 /* All the return strings MUST be freed */
 gchar* anjuta_res_get_pixmap_dir (void);
@@ -34,10 +44,21 @@ gchar* anjuta_res_get_doc_dir (void);
 gchar* anjuta_res_get_pixmap_file (const gchar* pixfile);
 gchar* anjuta_res_get_data_file (const gchar* pixfile);
 gchar* anjuta_res_get_help_file (const gchar* helpfile);
-gchar* anjuta_res_get_help_file_locale (const gchar* helpfile, const gchar* locale);
+gchar* anjuta_res_get_help_file_locale (const gchar* helpfile,
+										const gchar* locale);
 gchar* anjuta_res_get_doc_file (const gchar* docfile);
 
+/* File type icons 16x16; Do not unref the returned value */
+GdkPixbuf * anjuta_res_get_icon_for_file (PropsID props, const gchar *filename);
+
 /* System Help */
 void anjuta_res_help_search (const gchar * word);
 
+/* URI handler -- wrapped over gnome_url_show() */
+void anjuta_res_url_show(const char *url);
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* _RESOURCES_H_ */
