@@ -22,14 +22,22 @@
 
 #include "project_dbase.h"
 
-gboolean
-source_write_generic_main_c (ProjectDBase *data);
+#define CPP_HELLO_WORLD		"#include <iostream>\n"					\
+							"int main()\n{\n"						\
+							"\tstd::cout << \"Hello world\" << std::endl;\n" \
+							"\treturn 0;\n}\n\n"
 
-gboolean
-source_write_libglade_main_c (ProjectDBase *data);
+#define C_HELLO_WORLD		"#include <stdio.h>\n"					\
+							"int main()\n{\n"						\
+							"\tprintf(\"Hello world\\n\");\n"		\
+							"\treturn (0);\n}\n\n"
 
-gboolean
-source_write_build_files (ProjectDBase * data);
+gboolean source_write_generic_main_c (ProjectDBase *data);
+gboolean source_write_wxwin_main_c (ProjectDBase *data);
+gboolean source_write_xwin_main_c (ProjectDBase *data);
+gboolean source_write_xwindockapp_main_c (ProjectDBase *data);
+gboolean source_write_libglade_main_c (ProjectDBase *data);
+gboolean source_write_libglade2_main_c (ProjectDBase *data);
+gboolean source_write_build_files (ProjectDBase * data);
 
 #endif	/* __SOURCE_H__ */
-
