@@ -20,52 +20,20 @@
 #ifndef _STACK_TRACE_H_
 #define _STACK_TRACE_H_
 
-#include <gnome.h>
-#include "properties.h"
+#include <glib.h>
+#include <gtk/gtkwidget.h>
 
-typedef struct _StackTraceGui StackTraceGui;
 typedef struct _StackTrace StackTrace;
 
-struct _StackTraceGui
-{
-  GtkWidget *window;
-  GtkWidget *clist;
-  GtkWidget *menu;
-  GtkWidget *menu_set;
-  GtkWidget *menu_info;
-  GtkWidget *menu_update;
-  GtkWidget *menu_view;
-};
-
-struct _StackTrace
-{
-  StackTraceGui widgets;
-  gint current_index;
-  gint current_frame;
-  gboolean is_showing;
-  gint win_pos_x, win_pos_y, win_width, win_height;
-};
-
 StackTrace *stack_trace_new (void);
 
-void create_stack_trace_gui (StackTrace * st);
+/* Getters */
+GtkWidget *stack_trace_get_treeview (StackTrace *st);
 
-void stack_trace_clear (StackTrace * st);
-
-void stack_trace_update (GList * lines, gpointer st);
-
-void add_frame (StackTrace * st, gchar *);
-
-void stack_trace_update_controls (StackTrace * st);
-
-void stack_trace_destroy (StackTrace * st);
-
-gboolean stack_trace_save_yourself (StackTrace * st, FILE * stream);
-
-gboolean stack_trace_load_yourself (StackTrace * st, PropsID props);
-
-void stack_trace_show (StackTrace * st);
-
-void stack_trace_hide (StackTrace * st);
+void stack_trace_clear (StackTrace *st);
+void stack_trace_set_frame (StackTrace *st, gint frame);
+void stack_trace_update (GList *lines, gpointer st);
+void stack_trace_update_controls (StackTrace *st);
+void stack_trace_destroy (StackTrace *st);
 
 #endif
