@@ -19,16 +19,21 @@
 #ifndef _TEXT_EDITOR_MENU_H_
 #define _TEXT_EDITOR_MENU_H_
 
+#include <gtk/gtk.h>
+
 typedef struct _TextEditorMenu TextEditorMenu;
 struct _TextEditorMenu
 {
 	GtkWidget *GUI;
-	GtkWidget *copy;
 	GtkWidget *cut;
+	GtkWidget *copy;
+	GtkWidget *paste;
+	GtkWidget *context_help;
 	GtkWidget *autoformat;
 	GtkWidget *swap;
 	GtkWidget *functions;
 	GtkWidget *debug;
+	GtkWidget *docked;
 };
 
 void create_text_editor_menu_gui (TextEditorMenu *);
