@@ -30,14 +30,13 @@
 #include "text_editor.h"
 #include "mainmenu_callbacks.h"
 #include "toolbar_callbacks.h"
-#include "messagebox.h"
 #include "debugger.h"
-
+#include "search_incremental.h"
 
 void
 on_toolbar_new_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.file.new_file),
 				 "activate");
 }
@@ -46,7 +45,7 @@ on_toolbar_new_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_open_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.file.open_file),
 				 "activate");
 }
@@ -55,7 +54,7 @@ on_toolbar_open_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_save_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.file.save_file),
 				 "activate");
 }
@@ -63,7 +62,7 @@ on_toolbar_save_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_save_all_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.file.save_all_file),
 				 "activate");
 }
@@ -71,7 +70,7 @@ on_toolbar_save_all_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_close_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.file.close_file),
 				 "activate");
 }
@@ -80,7 +79,7 @@ on_toolbar_close_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_reload_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.file.reload_file),
 				 "activate");
 }
@@ -89,7 +88,7 @@ on_toolbar_reload_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_undo_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.edit.undo),
 				 "activate");
 }
@@ -98,7 +97,7 @@ on_toolbar_undo_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_redo_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.edit.redo),
 				 "activate");
 }
@@ -106,7 +105,7 @@ on_toolbar_redo_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_print_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.file.print),
 				 "activate");
 }
@@ -114,49 +113,36 @@ on_toolbar_print_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_detach_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.format.detach),
 				 "activate");
 }
 
+gboolean
+on_toolbar_find_incremental_start (GtkEntry *entry, GdkEvent *e, gpointer user_data)
+{
+	toolbar_search_incremental_start ();
+	return FALSE;
+}
+
+gboolean
+on_toolbar_find_incremental_end (GtkEntry *entry,
+	GdkEvent *e, gpointer user_data)
+{
+	toolbar_search_incremental_end ();
+	return FALSE;
+}
+
+void
+on_toolbar_find_incremental (GtkEntry *entry, gpointer user_data)
+{
+	toolbar_search_incremental ();
+}
+
 void
 on_toolbar_find_clicked (GtkButton * button, gpointer user_data)
 {
-	TextEditor *te;
-	gchar *string, *string1;
-	gchar buff[512];
-	gint ret;
-
-	te = anjuta_get_current_text_editor ();
-	if (!te)
-		return;
-	string1 =
-		gtk_entry_get_text (GTK_ENTRY
-				    (app->widgets.toolbar.main_toolbar.
-				     find_entry));
-	if (!string1 || strlen (string1) == 0)
-		return;
-	string = g_strdup (string1);
-	app->find_replace->find_text->find_history =
-		update_string_list (app->find_replace->find_text->
-				    find_history, string, COMBO_LIST_LENGTH);
-	gtk_combo_set_popdown_strings (GTK_COMBO
-				       (app->widgets.toolbar.main_toolbar.
-					find_combo),
-				       app->find_replace->find_text->
-				       find_history);
-
-	ret = text_editor_find (te, string,
-				TEXT_EDITOR_FIND_SCOPE_CURRENT,
-				app->find_replace->find_text->forward,
-				app->find_replace->find_text->regexp,
-				app->find_replace->find_text->ignore_case,
-				app->find_replace->find_text->whole_word);
-
-	sprintf (buff, _("The match \"%s\" was not found from the current location"), string);
-	if (ret < 0)
-		anjuta_error (buff);
-	g_free (string);
+	toolbar_search_clicked_cb ();
 }
 
 void
@@ -164,7 +150,7 @@ on_toolbar_goto_clicked (GtkButton * button, gpointer user_data)
 {
 	TextEditor *te;
 	guint line;
-	gchar *line_ascii;
+	const gchar *line_ascii;
 
 	te = anjuta_get_current_text_editor ();
 	if (te)
@@ -176,7 +162,7 @@ on_toolbar_goto_clicked (GtkButton * button, gpointer user_data)
 		if (strlen (line_ascii) == 0)
 			return;
 		line = atoi (line_ascii);
-		if (text_editor_goto_line (te, line, TRUE) == FALSE)
+		if (text_editor_goto_line (te, line, TRUE, TRUE) == FALSE)
 		{
 			anjuta_error (_("There is line number %d in \"%s\"."),
 				 line, te->filename);
@@ -188,7 +174,7 @@ void
 on_toolbar_tag_clicked (GtkButton * button, gpointer user_data)
 {
 	TextEditor *te;
-	gchar *string;
+	const gchar *string;
 
 	te = anjuta_get_current_text_editor ();
 	if (!te)
@@ -204,7 +190,7 @@ on_toolbar_tag_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_project_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.view.project_listing),
 				 "activate");
 }
@@ -212,7 +198,7 @@ on_toolbar_project_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_messages_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.view.messages),
 				 "activate");
 }
@@ -221,15 +207,13 @@ on_toolbar_messages_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_help_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
-				 (app->widgets.menubar.help.context_help),
-				 "activate");
+	on_context_help_activate(NULL, NULL);
 }
 
 void
 on_toolbar_open_project_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.file.open_project),
 				 "activate");
 }
@@ -237,7 +221,7 @@ on_toolbar_open_project_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_save_project_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.file.save_project),
 				 "activate");
 }
@@ -245,7 +229,7 @@ on_toolbar_save_project_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_close_project_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.file.close_project),
 				 "activate");
 }
@@ -253,7 +237,7 @@ on_toolbar_close_project_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_compile_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.build.compile),
 				 "activate");
 }
@@ -261,7 +245,7 @@ on_toolbar_compile_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_configure_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.build.configure),
 				 "activate");
 }
@@ -269,7 +253,7 @@ on_toolbar_configure_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_build_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.build.build),
 				 "activate");
 }
@@ -277,7 +261,7 @@ on_toolbar_build_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_build_all_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.build.build_all),
 				 "activate");
 }
@@ -285,7 +269,7 @@ on_toolbar_build_all_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_exec_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.build.execute),
 				 "activate");
 }
@@ -293,7 +277,7 @@ on_toolbar_exec_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_debug_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.debug.start_debug),
 				 "activate");
 }
@@ -301,7 +285,7 @@ on_toolbar_debug_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_stop_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.build.stop_build),
 				 "activate");
 }
@@ -309,7 +293,7 @@ on_toolbar_stop_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_go_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.debug.cont),
 				 "activate");
 }
@@ -317,7 +301,7 @@ on_toolbar_go_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_run_to_cursor_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.debug.run_to_cursor),
 				 "activate");
 }
@@ -325,7 +309,7 @@ on_toolbar_run_to_cursor_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_step_in_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.debug.step_in),
 				 "activate");
 }
@@ -334,7 +318,7 @@ on_toolbar_step_in_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_step_out_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.debug.step_out),
 				 "activate");
 }
@@ -342,7 +326,7 @@ on_toolbar_step_out_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_step_over_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.debug.step_over),
 				 "activate");
 }
@@ -350,31 +334,32 @@ on_toolbar_step_over_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_toggle_bp_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.debug.tog_break),
 				 "activate");
 }
 
+/*
 void
 on_toolbar_watch_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.view.variable_watch),
 				 "activate");
 }
-
+*/
 void
-on_toolbar_stack_clicked (GtkButton * button, gpointer user_data)
+on_toolbar_signals_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
-				 (app->widgets.menubar.view.program_stack),
+	g_signal_emit_by_name (G_OBJECT
+				 (app->widgets.menubar.view.signals),
 				 "activate");
 }
 
 void
 on_toolbar_registers_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.view.registers),
 				 "activate");
 }
@@ -382,7 +367,7 @@ on_toolbar_registers_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_frame_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.debug.info_frame),
 				 "activate");
 }
@@ -390,7 +375,7 @@ on_toolbar_frame_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_inspect_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.debug.inspect),
 				 "activate");
 }
@@ -398,7 +383,7 @@ on_toolbar_inspect_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_interrupt_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.debug.interrupt),
 				 "activate");
 }
@@ -406,7 +391,7 @@ on_toolbar_interrupt_clicked (GtkButton * button, gpointer user_data)
 void
 on_toolbar_debug_stop_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.debug.stop),
 				 "activate");
 }
@@ -422,7 +407,7 @@ on_browser_wizard_clicked (GtkButton * button, gpointer user_data)
 void
 on_browser_toggle_bookmark_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.bookmark.toggle),
 				 "activate");
 }
@@ -430,7 +415,7 @@ on_browser_toggle_bookmark_clicked (GtkButton * button, gpointer user_data)
 void
 on_browser_first_bookmark_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.bookmark.first),
 				 "activate");
 }
@@ -438,7 +423,7 @@ on_browser_first_bookmark_clicked (GtkButton * button, gpointer user_data)
 void
 on_browser_prev_bookmark_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.bookmark.prev),
 				 "activate");
 }
@@ -446,7 +431,7 @@ on_browser_prev_bookmark_clicked (GtkButton * button, gpointer user_data)
 void
 on_browser_next_bookmark_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.bookmark.next),
 				 "activate");
 }
@@ -454,7 +439,7 @@ on_browser_next_bookmark_clicked (GtkButton * button, gpointer user_data)
 void
 on_browser_last_bookmark_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.bookmark.last),
 				 "activate");
 }
@@ -462,7 +447,7 @@ on_browser_last_bookmark_clicked (GtkButton * button, gpointer user_data)
 void
 on_browser_prev_mesg_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.edit.goto_prev_mesg),
 				 "activate");
 }
@@ -470,7 +455,7 @@ on_browser_prev_mesg_clicked (GtkButton * button, gpointer user_data)
 void
 on_browser_next_mesg_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.edit.goto_next_mesg),
 				 "activate");
 }
@@ -478,7 +463,7 @@ on_browser_next_mesg_clicked (GtkButton * button, gpointer user_data)
 void
 on_browser_block_start_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.edit.goto_block_start),
 				 "activate");
 }
@@ -486,7 +471,7 @@ on_browser_block_start_clicked (GtkButton * button, gpointer user_data)
 void
 on_browser_block_end_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.edit.goto_block_end),
 				 "activate");
 }
@@ -494,7 +479,7 @@ on_browser_block_end_clicked (GtkButton * button, gpointer user_data)
 void
 on_format_fold_toggle_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.format.toggle_fold),
 				 "activate");
 }
@@ -502,7 +487,7 @@ on_format_fold_toggle_clicked (GtkButton * button, gpointer user_data)
 void
 on_format_fold_open_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.format.open_folds),
 				 "activate");
 }
@@ -510,7 +495,7 @@ on_format_fold_open_clicked (GtkButton * button, gpointer user_data)
 void
 on_format_fold_close_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.format.close_folds),
 				 "activate");
 }
@@ -518,7 +503,7 @@ on_format_fold_close_clicked (GtkButton * button, gpointer user_data)
 void
 on_format_indent_inc_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.format.indent_inc),
 				 "activate");
 }
@@ -526,7 +511,7 @@ on_format_indent_inc_clicked (GtkButton * button, gpointer user_data)
 void
 on_format_indent_dcr_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.format.indent_dcr),
 				 "activate");
 }
@@ -534,7 +519,7 @@ on_format_indent_dcr_clicked (GtkButton * button, gpointer user_data)
 void
 on_format_indent_auto_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.format.indent),
 				 "activate");
 }
@@ -542,9 +527,9 @@ on_format_indent_auto_clicked (GtkButton * button, gpointer user_data)
 void
 on_format_indent_style_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_notebook_set_page (GTK_NOTEBOOK
+/*	gtk_notebook_set_page (GTK_NOTEBOOK
 			       (app->preferences->widgets.notebook), 5);
-	gtk_signal_emit_by_name (GTK_OBJECT
+*/	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.settings.preferences),
 				 "activate");
 }
@@ -552,7 +537,7 @@ on_format_indent_style_clicked (GtkButton * button, gpointer user_data)
 void
 on_format_block_select_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.edit.select_block),
 				 "activate");
 }
@@ -565,7 +550,7 @@ on_format_calltip_clicked (GtkButton * button, gpointer user_data)
 void
 on_format_autocomplete_clicked (GtkButton * button, gpointer user_data)
 {
-	gtk_signal_emit_by_name (GTK_OBJECT
+	g_signal_emit_by_name (G_OBJECT
 				 (app->widgets.menubar.edit.autocomplete),
 				 "activate");
 }
