@@ -37,8 +37,8 @@
 #include "properties.h"
 #include "resources.h"
 
-gchar *
-extract_filename (gchar * full_filename)
+const gchar *
+extract_filename (const gchar * full_filename)
 {
 
 	gint i;
@@ -51,7 +51,7 @@ extract_filename (gchar * full_filename)
 		return full_filename;
 	if (full_filename[length - 1] == '/')
 		return &full_filename[length];
-	for (i = strlen (full_filename) - 1; i >= 0; i--)
+	for (i = length - 1; i >= 0; i--)
 		if (full_filename[i] == '/')
 			break;
 	if (i == 0 && full_filename[0] != '/')
@@ -59,50 +59,63 @@ extract_filename (gchar * full_filename)
 	return &full_filename[++i];
 }
 
-#define NSTR(S) ((S)?(S):"NULL")
-gchar *
-resolved_file_name(gchar *full_filename)
+gchar*
+extract_directory(const gchar* full_filename)
 {
-	char real_filename[PATH_MAX], *final_filename;
+	gint i;
+	gint length;
+	gchar* dir;
+
 	if (!full_filename)
-		final_filename = NULL;
-	else if (realpath(full_filename, real_filename))
-		final_filename = g_strdup(real_filename);
-	else
-		final_filename = g_strdup(full_filename);
-#ifdef DEBUG
-	g_message("%s -> %s", NSTR(full_filename), NSTR(final_filename));
-#endif
-	return final_filename;
+		return NULL;
+	length = strlen (full_filename);
+	if (length == 0)
+		return g_strdup(full_filename);
+	if (full_filename[length - 1] == '/')
+	{
+		dir = g_new0(char, length - 1);
+		strncpy(dir, full_filename, length - 2);
+		return dir;
+	}
+	for (i = length - 1; i >= 0; i--)
+		if (full_filename[i] == '/')
+			break;
+	if (i == 0)
+		return g_strdup("");
+	dir = g_new0(char, i + 1);
+	strncpy(dir, full_filename, i);
+	dir[i] = '\0';
+	return dir;
 }
 
+
 GList *
-update_string_list (GList * list, gchar * string, gint length)
+update_string_list (GList *p_list, const gchar *p_str, gint length)
 {
 	gint i;
 	gchar *str;
-	if (!string)
-		return list;
-	for (i = 0; i < g_list_length (list); i++)
+	if (!p_str)
+		return p_list;
+	for (i = 0; i < g_list_length (p_list); i++)
 	{
-		str = (gchar *) g_list_nth_data (list, i);
+		str = (gchar *) g_list_nth_data (p_list, i);
 		if (!str)
 			continue;
-		if (strcmp (string, str) == 0)
+		if (strcmp (p_str, str) == 0)
 		{
-			list = g_list_remove (list, str);
-			list = g_list_prepend (list, str);
-			return list;
+			p_list = g_list_remove (p_list, str);
+			p_list = g_list_prepend (p_list, str);
+			return p_list;
 		}
 	}
-	list = g_list_prepend (list, g_strdup (string));
-	while (g_list_length (list) > length)
+	p_list = g_list_prepend (p_list, g_strdup (p_str));
+	while (g_list_length (p_list) > length)
 	{
-		str = g_list_nth_data (list, g_list_length (list) - 1);
-		list = g_list_remove (list, str);
+		str = g_list_nth_data (p_list, g_list_length (p_list) - 1);
+		p_list = g_list_remove (p_list, str);
 		g_free (str);
 	}
-	return list;
+	return p_list;
 }
 
 void
@@ -192,10 +205,10 @@ parse_error_line (const gchar * line, gchar ** filename, int *lineno)
 	return FALSE;
 }
 
-gchar *
-get_file_extension (gchar * file)
+const gchar *
+get_file_extension (const gchar * file)
 {
-	gchar *pos;
+	const gchar *pos;
 	if (!file)
 		return NULL;
 	pos = strrchr (file, '.');
@@ -205,9 +218,10 @@ get_file_extension (gchar * file)
 		return NULL;
 }
 
-FileExtType get_file_ext_type (gchar * file)
+FileExtType get_file_ext_type (const gchar * file)
 {
-	gchar *pos, *filetype_str;
+	const gchar *pos;
+	gchar *filetype_str;
 	FileExtType filetype;
 
 	if (!file)
@@ -217,8 +231,8 @@ FileExtType get_file_ext_type (gchar * file)
 		return FILE_TYPE_UNKNOWN;
 
 	filetype_str =
-		prop_get_new_expand (app->preferences->props, "filetype.",
-				     file);
+		prop_get_new_expand (ANJUTA_PREFERENCES (app->preferences)->props,
+							 "filetype.", file);
 	if (filetype_str == NULL)
 		return FILE_TYPE_UNKNOWN;
 
@@ -366,6 +380,14 @@ FileExtType get_file_ext_type (gchar * file)
 	{
 		filetype = FILE_TYPE_WSCRIPT;
 	}
+	else if (strcmp (filetype_str, "lisp") == 0)
+	{
+		filetype = FILE_TYPE_LISP;
+	}
+	else if (strcmp (filetype_str, "matlab") == 0)
+	{
+		filetype = FILE_TYPE_MATLAB;
+	}
 	else
 	{
 		filetype = FILE_TYPE_UNKNOWN;
@@ -377,7 +399,7 @@ FileExtType get_file_ext_type (gchar * file)
 gchar *
 get_a_tmp_file ()
 {
-	static gint count;
+	static gint count = 0;
 	gchar *filename;
 	filename =
 		g_strdup_printf ("%s/anjuta_%d.%d", app->dirs->tmp, count++,
@@ -699,7 +721,7 @@ file_is_sticky (const gchar * fn)
 }
 
 gboolean
-copy_file (gchar * src, gchar * dest, gboolean show_error)
+copy_file (const gchar * src, const gchar * dest, gboolean show_error)
 {
 	FILE *input_fp, *output_fp;
 	gchar buffer[FILE_BUFFER_SIZE];
@@ -722,7 +744,7 @@ copy_file (gchar * src, gchar * dest, gboolean show_error)
 		if( show_error)
 			anjuta_system_error (errno, _("Unable to create file: %s."), dest);
 		fclose (input_fp);
-		return TRUE;
+		return FALSE;
 	}
 	
 	for (;;)
@@ -763,7 +785,7 @@ update_gtk ()
 {
 	/* Do not update gtk when launcher is busy */
 	/* This will freeze the application till the launcher is done */
-	if (launcher_is_busy () == TRUE)
+	if (anjuta_launcher_is_busy (app->launcher) == TRUE)
 		return;
 	if (app->auto_gtk_update == FALSE)
 		return;
@@ -777,16 +799,14 @@ void
 entry_set_text_n_select (GtkWidget * entry, gchar * text,
 			 gboolean use_selection)
 {
-	gchar *chars;
-
 	if (!entry)
 		return;
 	if (GTK_IS_ENTRY (entry) == FALSE)
 		return;
 
-	chars = anjuta_get_current_selection ();
 	if (use_selection)
 	{
+		gchar *chars = anjuta_get_current_selection ();
 		if (chars)
 		{
 			gtk_entry_set_text (GTK_ENTRY (entry), chars);
@@ -794,6 +814,8 @@ entry_set_text_n_select (GtkWidget * entry, gchar * text,
 						    strlen (gtk_entry_get_text
 							    (GTK_ENTRY
 							     (entry))));
+
+			g_free (chars);
 		}
 		else
 		{
@@ -813,12 +835,10 @@ entry_set_text_n_select (GtkWidget * entry, gchar * text,
 					    strlen (gtk_entry_get_text
 						    (GTK_ENTRY (entry))));
 	}
-	if (chars)
-		g_free (chars);
 }
 
 gboolean
-force_create_dir (gchar * d)
+force_create_dir (const gchar * d)
 {
 	if (file_is_directory (d))
 		return TRUE;
@@ -827,34 +847,34 @@ force_create_dir (gchar * d)
 	return TRUE;
 }
 
-GdkFont *
+PangoFontDescription *
 get_fixed_font ()
 {
-	static GdkFont *font;
+	static PangoFontDescription *font_desc;
 	static gint done;
 
 	if (done)
-		return font;
-	font =
-		gdk_font_load
-		("-misc-fixed-medium-r-semicondensed-*-*-120-*-*-c-*-iso8859-9");
+		return font_desc;
+	font_desc = pango_font_description_from_string ("misc medium 12");
 	done = 1;
-	if (font)
+	if (font_desc)
 	{
-		gdk_font_ref (font);
-		return font;
+		// pango_font_ref (font);
+		return font_desc;
 	}
 	g_warning ("Cannot load fixed(misc) font. Using default font.");
 	return NULL;
 }
 
 gchar *
-remove_white_spaces (gchar * text)
+remove_white_spaces (const gchar * text)
 {
 	guint src_count, dest_count, tab_count;
 	gchar buff[2048];	/* Let us hope that it does not overflow */
 
-	tab_count = preferences_get_int (app->preferences, TAB_SIZE);
+	tab_count =
+		anjuta_preferences_get_int (ANJUTA_PREFERENCES (app->preferences),
+									TAB_SIZE);
 	dest_count = 0;
 	for (src_count = 0; src_count < strlen (text); src_count++)
 	{
@@ -910,7 +930,7 @@ gboolean widget_is_child (GtkWidget * parent, GtkWidget * child)
 	{
 		GList *children;
 		GList *node;
-		children = gtk_container_children (GTK_CONTAINER (parent));
+		children = gtk_container_get_children (GTK_CONTAINER (parent));
 		node = children;
 		while (node)
 		{
@@ -926,8 +946,9 @@ gboolean widget_is_child (GtkWidget * parent, GtkWidget * child)
 	return FALSE;
 }
 
+/* GList of strings operations */
 GList *
-glist_from_string (gchar * string)
+glist_from_string (const gchar *string)
 {
 	gchar *str, *temp, buff[256];
 	GList *list;
@@ -972,31 +993,57 @@ glist_from_string (gchar * string)
 	return list;
 }
 
+/* Get the list of strings as GList from a property value.
+   Strings are splitted from white spaces */
 GList *
-glist_from_data (guint props, gchar * id)
+glist_from_data (guint props, const gchar *id)
 {
 	gchar *str;
 	GList *list;
 
 	str = prop_get (props, id);
 	list = glist_from_string (str);
+	g_free(str);
 	return list;
 }
 
+/* Prefix the strings */
 void
-glist_strings_free (GList * list)
+glist_strings_prefix (GList * list, const gchar *prefix)
 {
 	GList *node;
 	node = list;
+	
+	g_return_if_fail (prefix != NULL);
 	while (node)
 	{
-		if (node->data)
-			g_free (node->data);
+		gchar* tmp;
+		tmp = node->data;
+		node->data = g_strconcat (prefix, tmp, NULL);
+		if (tmp) g_free (tmp);
 		node = g_list_next (node);
 	}
-	g_list_free (list);
 }
 
+/* Suffix the strings */
+void
+glist_strings_sufix (GList * list, const gchar *sufix)
+{
+	GList *node;
+	node = list;
+	
+	g_return_if_fail (sufix != NULL);
+	while (node)
+	{
+		gchar* tmp;
+		tmp = node->data;
+		node->data = g_strconcat (tmp, sufix, NULL);
+		if (tmp) g_free (tmp);
+		node = g_list_next (node);
+	}
+}
+
+/* Duplicate list of strings */
 GList*
 glist_strings_dup (GList * list)
 {
@@ -1016,56 +1063,40 @@ glist_strings_dup (GList * list)
 	return new_list;
 }
 
-void
-string_free(gchar* str)
+/* Dedup a list of paths - duplicates are removed from the tail.
+** Useful for deduping Recent Files and Recent Projects */
+GList *glist_path_dedup(GList *list)
 {
-	if (str) g_free(str);
-}
-
-void
-string_assign (gchar ** string, gchar * value)
-{
-	if (*string)
-		g_free (*string);
-	*string = NULL;
-	if (value)
-		*string = g_strdup (value);
-}
-void
-glist_strings_prefix (GList * list, gchar *prefix)
-{
-	GList *node;
-	node = list;
-	
-	g_return_if_fail (prefix != NULL);
-	while (node)
+	GList *nlist = NULL, *tmp, *tmp1;
+	gchar *path;
+	struct stat s;
+	for (tmp = list; tmp; tmp = g_list_next(tmp))
 	{
-		gchar* tmp;
-		tmp = node->data;
-		node->data = g_strconcat (prefix, tmp, NULL);
-		if (tmp) g_free (tmp);
-		node = g_list_next (node);
+		path = tm_get_real_path((const gchar *) tmp->data);
+		if (0 != stat(path, &s))
+		{
+			g_free(path);
+		}
+		else
+		{
+			for (tmp1 = nlist; tmp1; tmp1 = g_list_next(tmp1))
+			{
+				if (0 == strcmp((const char *) tmp1->data, path))
+				{
+					g_free(path);
+					path = NULL;
+					break;
+				}
+			}
+			if (path)
+			nlist = g_list_prepend(nlist, path);
+		}
 	}
+	glist_strings_free(list);
+	nlist = g_list_reverse(nlist);
+	return nlist;
 }
 
-void
-glist_strings_sufix (GList * list, gchar *sufix)
-{
-	GList *node;
-	node = list;
-	
-	g_return_if_fail (sufix != NULL);
-	while (node)
-	{
-		gchar* tmp;
-		tmp = node->data;
-		node->data = g_strconcat (tmp, sufix, NULL);
-		if (tmp) g_free (tmp);
-		node = g_list_next (node);
-	}
-}
-
-
 static gint
 sort_node (gchar* a, gchar *b)
 {
@@ -1075,14 +1106,41 @@ sort_node (gchar* a, gchar *b)
 	return strcmp (a, b);
 }
 
+/* Sort the list alphabatically */
 GList*
 glist_strings_sort (GList * list)
 {
 	return g_list_sort(list, (GCompareFunc)sort_node);
 }
 
+/* Free the strings and GList */
+void
+glist_strings_free (GList * list)
+{
+	GList *node;
+	node = list;
+	while (node)
+	{
+		if (node->data)
+			g_free (node->data);
+		node = g_list_next (node);
+	}
+	g_list_free (list);
+}
+
+/* Assign a value to a string */
+void
+string_assign (gchar ** string, const gchar *value)
+{
+	if (*string)
+		g_free (*string);
+	*string = NULL;
+	if (value)
+		*string = g_strdup (value);
+}
+
 gchar*
-get_swapped_filename(gchar* filename)
+get_swapped_filename(const gchar* filename)
 {
 	size_t len;
 	gchar *newfname;
@@ -1148,47 +1206,105 @@ get_swapped_filename(gchar* filename)
 	g_free(newfname);	
 	return NULL;
 }
-gboolean
-move_file_if_not_same (gchar* src, gchar* dest)
-{
-	gboolean same;
 
+gboolean
+move_file_if_not_same (const gchar* src, const gchar* dest)
+{
 	g_return_val_if_fail (src != NULL, FALSE);
 	g_return_val_if_fail (dest != NULL, FALSE);
 
-	if (anjuta_is_installed ("cmp", FALSE) == TRUE)
-	{
-		pid_t pid;
-		gint status;
-		if ((pid=fork())==0)
-		{
-			execlp ("cmp", "cmp", "-s", src, dest, NULL);
-			g_error ("Cannot execute cmp");
-		}
-		waitpid (pid, &status, 0);
-		if (WEXITSTATUS(status)==0)
-			same = TRUE;
-		else
-			same = FALSE;
-	}
-	else
-	{
-		same = FALSE;
-	}
 	/* rename () can't work with 2 different filesystems, so we need
 	 to use the couple {copy, remove}. Meanwhile, the files to copy
 	 are supposed to be small, so it would spend more time to try 
 	 rename () to know if a copy/deletion is needed */
-	
-	if (!same && !copy_file (src, dest, FALSE))
+
+	if (!compare_files(src, dest, FALSE) && !copy_file (src, dest, FALSE))
 		return FALSE;
-	
+
 	remove (src);
 	return TRUE;
 }
 
+gboolean
+compare_files (const gchar* file1, const gchar* file2, gboolean show_error)
+{
+	FILE *file_fp1, *file_fp2;
+	gchar buffer1[FILE_BUFFER_SIZE], buffer2[FILE_BUFFER_SIZE];
+	gint bytes_read1, bytes_read2;
+	gboolean same;
+
+	same = TRUE;	
+	
+	file_fp1 = fopen (file1, "rb");
+	if (file_fp1 == NULL)
+	{
+		if (show_error)
+			anjuta_system_error (errno, _("Unable to read file: %s."), file1);
+		return FALSE;
+	}
+	
+	file_fp2 = fopen (file2, "rb");
+	if (file_fp2 == NULL)
+	{
+		if (show_error)
+			anjuta_system_error(errno, _("Unable to read file: %s."), file2);
+		fclose(file_fp1);  // file1 was opened successfully, close it.
+		return FALSE;
+	}
+	
+	do
+	{
+		bytes_read1 = fread(buffer1, 1, FILE_BUFFER_SIZE, file_fp1);
+		bytes_read2 = fread(buffer2, 1, FILE_BUFFER_SIZE, file_fp2);
+		if ((ferror(file_fp1) != 0) || (ferror(file_fp2) != 0))
+		{
+			if (show_error)
+				anjuta_system_error(errno, 
+									_("Read-error while comparing %s with %s"),
+  							file1, file2);
+			fclose(file_fp2);
+			fclose(file_fp1);
+			return FALSE;
+		}
+		
+		// min-check neccessary because buffers uninited: filesize < buffersize => fail
+		if (memcmp(buffer1, buffer2, (bytes_read1 < bytes_read2) ? bytes_read1 : bytes_read2) != 0)
+		{                             
+			same = FALSE;
+			break;
+		}			
+	}
+	while ((bytes_read1 == bytes_read2) && (bytes_read1 == FILE_BUFFER_SIZE));
+	
+	fclose(file_fp2);
+	fclose(file_fp1);
+
+	return (bytes_read1 == bytes_read2) && same;
+}
+
+gboolean is_file_same(const gchar *a, const gchar *b)
+{
+	struct stat st_a,st_b;
+
+	if(stat(a, &st_a) == -1)
+	{
+		/* printf("WARNING: Unable to stat '%s'.", a);*/
+		return FALSE;
+	}
+	if(stat(b, &st_b) == -1)
+	{
+		/* printf("WARNING: Unable to stat '%s'.", b);*/
+		return FALSE;
+	}
+
+	if(st_a.st_ino == st_b.st_ino)
+		return TRUE;
+	else
+		return FALSE;
+}
+
 gchar*
-get_file_as_buffer (gchar* filename)
+get_file_as_buffer (const gchar* filename)
 {
 	struct stat s;
 	gchar *buff;
@@ -1215,18 +1331,20 @@ GList*
 scan_files_in_dir (const char *dir, int (*select)(const struct dirent *))
 {
 	struct dirent **namelist;
-	GList *files;
-	int n, i;
+	GList *files = NULL;
+	int n;
 
 	g_return_val_if_fail (dir != NULL, NULL);
-	
-	n = scandir (dir, &namelist, select, alphasort);
-	files = NULL;
-	if (n >= 0)
+	if (0 > (n = scandir (dir, &namelist, select, alphasort)))
+		return NULL;
+	else
 	{
-		for (i = 0; i < n; i++)
-			files = g_list_append (files, g_strdup (namelist[i]->d_name));
-		free (namelist);
+		while (n--)
+		{
+			files = g_list_append (files, g_strdup (namelist[n]->d_name));
+			free(namelist[n]);
+		}
+		free(namelist);
 	}
 	return files;
 }
@@ -1267,33 +1385,33 @@ select_only_file (const struct dirent *e)
 /* Create a new hbox with an image and a label packed into it
         * and return the box. */
 GtkWidget*
-create_xpm_label_box( GtkWidget *parent,
-                                 const gchar     *xpm_filename, gboolean gnome_pixmap,
-                                 const gchar     *label_text )
-       {
-           GtkWidget *box1;
-           GtkWidget *label;
-           GtkWidget *pixmap;
-
-           /* Create box for xpm and label */
-           box1 = gtk_hbox_new (FALSE, 0);
-
-           /* Now on to the xpm stuff */
-           pixmap = anjuta_res_get_pixmap_widget (parent, xpm_filename, gnome_pixmap);
-
-           /* Create a label for the button */
-           label = gtk_label_new (label_text);
-
-           /* Pack the pixmap and label into the box */
-           gtk_box_pack_start (GTK_BOX (box1),
-                               pixmap, FALSE, FALSE, 0);
-
-           gtk_box_pack_start (GTK_BOX (box1), label, FALSE, FALSE, 3);
-
-           gtk_widget_show(pixmap);
-           gtk_widget_show(label);
-
-           return(box1);
+create_xpm_label_box(GtkWidget *parent,
+					 const gchar     *xpm_filename, gboolean gnome_pixmap,
+					 const gchar     *label_text )
+{
+	GtkWidget *box1;
+	GtkWidget *label;
+	GtkWidget *pixmap;
+	
+	/* Create box for xpm and label */
+	box1 = gtk_hbox_new (FALSE, 0);
+	
+	/* Now on to the xpm stuff */
+	pixmap = anjuta_res_get_image (xpm_filename);
+	
+	/* Create a label for the button */
+	label = gtk_label_new (label_text);
+	
+	/* Pack the pixmap and label into the box */
+	gtk_box_pack_start (GTK_BOX (box1),
+					   pixmap, FALSE, FALSE, 0);
+	
+	gtk_box_pack_start (GTK_BOX (box1), label, FALSE, FALSE, 3);
+	
+	gtk_widget_show(pixmap);
+	gtk_widget_show(label);
+	
+	return(box1);
 }
 
 /* Excluding the final 0 */
@@ -1434,3 +1552,416 @@ gchar* GetStrCod( const gchar *szIn )
 	return szRet ;
 }
 
+gchar*
+anjuta_util_escape_quotes(const gchar* str)
+{
+	gchar buffer[2048];
+	gint idx;
+	const gchar *s = str;
+	
+	g_return_val_if_fail(str, NULL);
+	idx = 0;
+	
+	while(*s) {
+		if (idx > 2040)
+			break;
+		if (*s == '\"' || *s == '\'' || *s == '\\')
+			buffer[idx++] = '\\';
+		buffer[idx++] = *s;
+		s++;
+	}
+	buffer[idx] = '\0';
+	return g_strdup(buffer);
+}
+
+gchar *get_relative_file_name(const gchar *dir, const gchar *file)
+{
+	gchar *real_dir = tm_get_real_path(dir);
+	gchar *real_file = tm_get_real_path(file);
+	gchar *retval = NULL;
+
+	if (real_dir && real_file)
+	{
+		guint dir_len = strlen(real_dir);
+		if (0 == strncmp(real_file, real_dir, dir_len))
+			retval = g_strdup(real_file + dir_len + 1);
+	}
+	g_free(real_dir);
+	g_free(real_file);
+	return retval;
+}
+
+gboolean is_file_in_dir(const gchar *file, const gchar *dir)
+{
+	gboolean status = FALSE;
+
+	gchar *real_file_name = tm_get_real_path(file);
+	gchar *real_dir_name = tm_get_real_path(dir);
+	if (real_file_name && real_dir_name &&
+	  (0 == strncmp(real_file_name, real_dir_name, strlen(real_dir_name))))
+		status = TRUE;
+	g_free(real_file_name);
+	g_free(real_dir_name);
+	return status;
+}
+
+gint
+anjuta_util_kill (pid_t process_id, const gchar* signal)
+{
+	int status;
+	gchar *pid_str;
+	pid_t pid;
+	
+	pid_str = g_strdup_printf ("%d", process_id);
+	pid = fork();
+	if (pid == 0)
+	{
+		execlp ("kill", "kill", "-s", signal, pid_str, NULL);
+		g_warning (_("Can not execute command: \"%s\""), "kill");
+		_exit(1);
+	}
+	g_free (pid_str);
+	if (pid > 0) {
+		waitpid (pid, &status, 0);
+		return 0;
+	} else {
+		return -1;
+	}
+}
+
+GList*
+anjuta_util_parse_args_from_string (const gchar* string)
+{
+	gboolean escaped;
+	gchar    quote;
+	gchar    buffer[2048];
+	const gchar *s;
+	gint     idx;
+	GList* args = NULL;
+	
+	idx = 0;
+	escaped = FALSE;
+	quote = (gchar)-1;
+	s = string;
+	
+	while (*s) {
+		if (!isspace(*s))
+			break;
+		s++;
+	}
+
+	while (*s) {
+		if (escaped) {
+			/* The current char was escaped */
+			buffer[idx++] = *s;
+			escaped = FALSE;
+		} else if (*s == '\\') {
+			/* Current char is an escape */
+			escaped = TRUE;
+		} else if (*s == quote) {
+			/* Current char ends a quotation */
+			quote = (gchar)-1;
+			if (!isspace(*(s+1)) && (*(s+1) != '\0')) {
+				/* If there is no space after the quotation or it is not
+				   the end of the string */
+				g_warning ("Parse error while parsing program arguments");
+			}
+		} else if ((*s == '\"' || *s == '\'')) {
+			if (quote == (gchar)-1) {
+				/* Current char starts a quotation */
+				quote = *s;
+			} else {
+				/* Just a quote char inside quote */
+				buffer[idx++] = *s;
+			}
+		} else if (quote > 0){
+			/* Any other char inside quote */
+			buffer[idx++] = *s;
+		} else if (isspace(*s)) {
+			/* Any white space outside quote */
+			if (idx > 0) {
+				buffer[idx++] = '\0';
+				args = g_list_append (args, g_strdup (buffer));
+				idx = 0;
+			}
+		} else {
+			buffer[idx++] = *s;
+		}
+		s++;
+	}
+	if (idx > 0) {
+		/* There are chars in the buffer. Flush as the last arg */
+		buffer[idx++] = '\0';
+		args = g_list_append (args, g_strdup (buffer));
+		idx = 0;
+	}
+	if (quote > 0) {
+		g_warning ("Unclosed quotation encountered at the end of parsing");
+	}
+	return args;
+}
+
+/* Check which gnome-terminal is installed
+ Returns: 0 -- No gnome-terminal
+ Returns: 1 -- Gnome1 gnome-terminal
+ Returns: 2 -- Gnome2 gnome-terminal */
+
+gint 
+anjuta_util_check_gnome_terminal (void)
+{
+#ifdef DEBUG
+    gchar* term_command = "gnome-terminal --version";
+    gchar* term_command2 = "gnome-terminal --disable-factory --version";
+#else
+    gchar* term_command = "gnome-terminal --version > /dev/null 2> /dev/null";
+    gchar* term_command2 = "gnome-terminal --disable-factory --version > /dev/null 2> /dev/null";
+#endif
+    gint retval;
+    
+    retval = system (term_command);
+    
+    /* Command failed or gnome-terminal not found */
+    if (WEXITSTATUS(retval) != 0)
+        return 0;
+    
+    /* gnome-terminal found: Determine version 1 or 2 */
+    retval = system (term_command2);
+    
+    /* Command failed or gnome-terminal-2 not found */
+    if (WEXITSTATUS(retval) != 0)
+        return 1;
+    
+    /* gnome-terminal-2 found */
+    return 2;
+}
+
+int type_from_string(StringMap *map, const char *str)
+{
+	int i = 0;
+
+	while (-1 != map[i].type)
+	{
+		if (0 == strcmp(map[i].name, str))
+			return map[i].type;
+		++ i;
+	}
+	return -1;
+}
+
+const char *string_from_type(StringMap *map, int type)
+{
+	int i = 0;
+	while (-1 != map[i].type)
+	{
+		if (map[i].type == type)
+			return map[i].name;
+		++ i;
+	}
+	return "";
+}
+
+GList *glist_from_map(StringMap *map)
+{
+	GList *out_list = NULL;
+	int i = 0;
+	while (-1 != map[i].type)
+	{
+		out_list = g_list_append(out_list, map[i].name);
+		++ i;
+	}
+	return out_list;
+}
+
+static gint
+int_from_hex_digit (const gchar ch)
+{
+	if (isdigit (ch))
+		return ch - '0';
+	else if (ch >= 'A' && ch <= 'F')
+		return ch - 'A' + 10;
+	else if (ch >= 'a' && ch <= 'f')
+		return ch - 'a' + 10;
+	else
+		return 0;
+}
+
+void
+anjuta_util_color_from_string (const gchar * val, guint8 * r, guint8 * g, guint8 * b)
+{
+	*r = int_from_hex_digit (val[1]) * 16 + int_from_hex_digit (val[2]);
+	*g = int_from_hex_digit (val[3]) * 16 + int_from_hex_digit (val[4]);
+	*b = int_from_hex_digit (val[5]) * 16 + int_from_hex_digit (val[6]);
+}
+
+gchar *
+anjuta_util_string_from_color (guint8 r, guint8 g, guint8 b)
+{
+	gchar str[10];
+	guint32 num;
+
+	num = r;
+	num <<= 8;
+	num += g;
+	num <<= 8;
+	num += b;
+
+	sprintf (str, "#%06X", num);
+	return g_strdup (str);
+}
+
+gchar *
+anjuta_util_convert_to_utf8 (const gchar *str)
+{
+	GError *error = NULL;
+	gchar *utf8_msg_string = NULL;
+	
+	g_return_val_if_fail (str != NULL, NULL);
+	g_return_val_if_fail (strlen (str) > 0, NULL);
+	
+	if (g_utf8_validate(str, -1, NULL))
+	{
+		utf8_msg_string = g_strdup (str);
+	}
+	else
+	{
+		gsize rbytes, wbytes;
+		utf8_msg_string = g_locale_to_utf8 (str, -1, &rbytes, &wbytes, &error);
+		if (error != NULL) {
+			g_warning ("g_locale_to_utf8 failed: %s\n", error->message);
+			g_error_free (error);
+			g_free (utf8_msg_string);
+			return NULL;
+		}
+	}
+	return utf8_msg_string;
+}
+
+GtkWidget *
+anjuta_util_toolbar_append_button (GtkWidget *toolbar, const gchar *iconfile,
+					   const gchar *label, const gchar *tooltip,
+					   GtkSignalFunc callback, gpointer user_data)
+{
+	GtkWidget *icon = anjuta_res_get_image (iconfile);
+	GtkWidget *item =
+		gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
+					    GTK_TOOLBAR_CHILD_BUTTON, NULL,
+					    label, tooltip, NULL,
+					    icon, callback, user_data);
+	gtk_widget_ref (item);
+	gtk_widget_show (item);
+	return item;
+}
+
+GtkWidget *
+anjuta_util_toolbar_append_stock (GtkWidget *toolbar, const gchar *stock_icon,
+								const gchar *tooltip, GtkSignalFunc callback, 
+								gpointer user_data)
+{
+	GtkWidget *item;
+	item =
+		gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
+					    stock_icon,
+					    tooltip, NULL,
+					    callback, user_data, -1);
+	gtk_widget_ref (item);
+	gtk_widget_show (item);
+	return item;
+}
+
+GtkWidget* 
+anjuta_button_new_with_stock_image (const gchar* text, const gchar* stock_id)
+{
+	GtkWidget *button;
+	GtkStockItem item;
+	GtkWidget *label;
+	GtkWidget *image;
+	GtkWidget *hbox;
+	GtkWidget *align;
+
+	button = gtk_button_new ();
+
+ 	if (GTK_BIN (button)->child)
+    		gtk_container_remove (GTK_CONTAINER (button),
+				      GTK_BIN (button)->child);
+
+  	if (gtk_stock_lookup (stock_id, &item))
+    	{
+      		label = gtk_label_new_with_mnemonic (text);
+
+		gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (button));
+      
+		image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_BUTTON);
+      		hbox = gtk_hbox_new (FALSE, 2);
+
+      		align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+      
+      		gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+      		gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+      
+      		gtk_container_add (GTK_CONTAINER (button), align);
+      		gtk_container_add (GTK_CONTAINER (align), hbox);
+      		gtk_widget_show_all (align);
+
+      		return button;
+    	}
+
+      	label = gtk_label_new_with_mnemonic (text);
+      	gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (button));
+  
+  	gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0.5);
+
+  	gtk_widget_show (label);
+  	gtk_container_add (GTK_CONTAINER (button), label);
+
+	return button;
+}
+
+GtkWidget*
+anjuta_dialog_add_button (GtkDialog *dialog, const gchar* text,
+						  const gchar* stock_id, gint response_id)
+{
+	GtkWidget *button;
+	
+	g_return_val_if_fail (GTK_IS_DIALOG (dialog), NULL);
+	g_return_val_if_fail (text != NULL, NULL);
+	g_return_val_if_fail (stock_id != NULL, NULL);
+
+	button = anjuta_button_new_with_stock_image (text, stock_id);
+	g_return_val_if_fail (button != NULL, NULL);
+
+	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+
+	gtk_widget_show (button);
+
+	gtk_dialog_add_action_widget (dialog, button, response_id);	
+
+	return button;
+}
+
+pid_t
+anjuta_execute_shell (const gchar *dir, const gchar *command)
+{
+	pid_t pid;
+	gchar *shell;
+	
+	g_return_val_if_fail (command != NULL, -1);
+	
+	shell = gnome_util_user_shell ();
+	pid = fork();
+	if (pid == 0)
+	{
+		if(dir)
+		{
+			force_create_dir (dir);
+			chdir (dir);
+		}
+		execlp (shell, shell, "-c", command, NULL);
+		g_warning (_("Cannot execute command: %s (using shell %s)\n"), command, shell);
+		_exit(1);
+	}
+	if (pid < 0)
+		g_warning (_("Cannot execute command %s (using shell %s)\n"), command, shell);
+	g_free (shell);
+	// Anjuta will take care of child exit automatically.
+	return pid;
+}
