@@ -19,11 +19,15 @@
 #ifndef _UTILITIES_H_
 #define _UTILITIES_H_
 
+#define FREE(x) if (x) g_free(x), x = NULL
+
 #include <dirent.h>
+#include <stdio.h>
 
 #define COMBO_LIST_LENGTH \
-		preferences_get_int (app->preferences, MAXIMUM_COMBO_HISTORY)
+		anjuta_preferences_get_int (app->preferences, MAXIMUM_COMBO_HISTORY)
 
+#define _STR(S) ((S)?(S):"")
 
 /* Any addition to this enum must be synced with the
 function get_file_ext_type() */
@@ -67,6 +71,8 @@ enum _FileExtType
 	FILE_TYPE_BAAN,
 	FILE_TYPE_ADA,
 	FILE_TYPE_WSCRIPT,
+	FILE_TYPE_LISP,
+	FILE_TYPE_MATLAB,
 	FILE_TYPE_END_MARK
 };
 
@@ -77,20 +83,17 @@ typedef enum _FileExtType FileExtType;
 /****************************************************************************/
 
 /* Removes while spaces in the text */
-gchar* remove_white_spaces(gchar* text);
+gchar* remove_white_spaces(const gchar* text);
 
 /* Gets the swapped (c/h) file names */
-gchar* get_swapped_filename(gchar* filename);
+gchar* get_swapped_filename(const gchar* filename);
 
 /* Get a unique temporary filename */
 /* This filename is not valid beyond the present session */
 gchar* get_a_tmp_file(void);
 
-/* Gets a resolved file name with extra '/', '..', etc. removed */
-gchar *resolved_file_name(gchar *full_filename);
-
 /* Retruns the contents of the file a buffer */
-gchar* get_file_as_buffer (gchar* filename);
+gchar* get_file_as_buffer (const gchar* filename);
 
 /* Creates a space separated string from the list of strings */
 gchar* string_from_glist (GList* list);
@@ -105,10 +108,8 @@ gboolean parse_error_line(const gchar *line, gchar **filename, int *lineno);
 /*  Functions that do not dynamic allocate memory. Return value should not be g_freed */
 /***********************************************************************************/
 
-void string_free(gchar* str);
-
 /* Get a fixed font */
-GdkFont* get_fixed_font (void);
+PangoFontDescription* get_fixed_font (void);
 
 /* Rather tough */
 gboolean read_string(FILE* stream, gchar* token, gchar **str);
@@ -117,18 +118,21 @@ gboolean write_string(FILE* stream, gchar* token, gchar *str);
 gboolean write_line(FILE* stream, gchar *str);
 
 /* Get the filename */
-char* extract_filename(char* full_filename);
+const gchar* extract_filename (const gchar* full_filename);
+
+/* Get directory of file, return value must be g_freed */
+gchar* extract_directory (const gchar* full_filename);
 
 /* Adds the given string in the list, if it does not already exist. */
 /* The added string will come at the top of the list */
 /* The list will be then truncated to (length) items only */
-GList* update_string_list(GList *list, gchar *string, gint length);
+GList* update_string_list(GList *p_list, const gchar *p_str, gint length);
 
 /* Gets the extension of the file */
-gchar* get_file_extension(gchar* file);
+const gchar* get_file_extension(const gchar* file);
 
 /* Gets the file extension type */
-FileExtType get_file_ext_type(gchar* file);
+FileExtType get_file_ext_type(const gchar* file);
 
 /*
 void fast_widget_repaint(GtkWidget* w);
@@ -157,12 +161,20 @@ gboolean file_is_sticky(const gchar * fn);
 
 /* copies src to dest */
 /* if (show_error) is true, then a dialog box will be shown for the error */
-gboolean copy_file(gchar* src, gchar* dest, gboolean show_error);
+gboolean copy_file(const gchar* src, const gchar* dest, gboolean show_error);
+
+/* bytewise compares two files and returns true for same or false for */
+/* different or error */
+/* if (show_error) is true, then a dialog box will be shown for the error */
+gboolean compare_files(const gchar* file1, const gchar* file2, gboolean show_error);
 
 /* (src) will be renamed, ie moved, to (dest) if they are not same */
 /* Used to avoid unnecessary change in modified time of the dest file */
 /* Which may result in recompilation, etc */
-gboolean move_file_if_not_same (gchar* src, gchar* dest);
+gboolean move_file_if_not_same (const gchar* src, const gchar* dest);
+
+/* Check if two files/dirs/etc are the same one */
+gboolean is_file_same(const gchar *a, const gchar *b);
 
 /* Update, flush and paint all gtk widgets */
 void update_gtk(void);
@@ -173,7 +185,7 @@ void entry_set_text_n_select(GtkWidget* w, gchar* chars, gboolean use_selection)
 
 /* Creates a dir, if it already doesnot exists */
 /* All the required parent dirs are also created if they don't exist */
-gboolean force_create_dir(gchar* d);
+gboolean force_create_dir(const gchar* d);
 
 /* Checks if the (parent) is parent widget of the (child)*/
 gboolean widget_is_child(GtkWidget *parent, GtkWidget* child);
@@ -181,7 +193,7 @@ gboolean widget_is_child(GtkWidget *parent, GtkWidget* child);
 /* Fast string assignment to avoid memory leaks */
 /* Assigns a new string (value), if not NULL, with g_strdup() to (*string) */
 /* The previous string, if not NULL, is g_freed */
-void string_assign (gchar** string, gchar* value);
+void string_assign (gchar** string, const gchar* value);
 
 /* Returns true if the e->d_name is a regular file */
 /* Used as the callback for scandir() */
@@ -191,22 +203,22 @@ int select_only_file (const struct dirent *e);
 /*  Functions that operate on list of strings. */
 /***********************************************/
 void glist_strings_free(GList* list);
-void glist_strings_prefix (GList * list, gchar *prefix);
-void glist_strings_sufix (GList * list, gchar *sufix);
+void glist_strings_prefix (GList * list, const gchar *prefix);
+void glist_strings_sufix (GList * list, const gchar *sufix);
 GList* glist_strings_sort (GList * list);
 
 /**********************************************************/
 /* Both the returned glist and the data should be g_freed */
+/* Call g_list_strings_free() to do that.                 */
 /**********************************************************/
-GList* glist_from_data(guint props, gchar* id);
-GList* glist_from_string(gchar* id);
+GList* glist_from_data (guint props, const gchar* id);
+GList* glist_from_string (const gchar* id);
 GList* glist_strings_dup (GList * list);
 
 /* Returns the alphabatically sorted list for files in dir */
 /* select is the func that approves (returning 1)each file */
 GList* scan_files_in_dir (const char *dir, int (*select)(const struct dirent *));
 
-
 /*******************************************************************/
 /* In this case only GList must be freed and not the data          */
 /* Because output data are the input data. Only GList is allocated */
@@ -216,9 +228,9 @@ GList* remove_blank_lines(GList* lines);
 /* Create a new hbox with an image and a label packed into it
         * and return the box. */
 GtkWidget*
-create_xpm_label_box( GtkWidget *parent,
-                                 const gchar     *xpm_filename, gboolean gnome_pixmap,
-                                 const gchar     *label_text );
+create_xpm_label_box(GtkWidget *parent,
+					 const gchar     *xpm_filename, gboolean gnome_pixmap,
+					 const gchar     *label_text );
 /* Excluding the final 0 */
 gint calc_string_len( const gchar *szStr );
 gint calc_gnum_len( void /*const gint iVal*/ );
@@ -237,4 +249,78 @@ gchar *WriteBufS( gchar* szDst, const gchar* szVal );
 void
 free_string_list ( GList * pList );
 
+/********************************************************
+ * Escapes single and double quotes in the given string *
+ * Return string must be g_freed()                      *
+ ********************************************************/
+gchar* anjuta_util_escape_quotes(const gchar* str);
+
+/********************************************************
+ * Use this function instead of kill() to send a signal *
+ * to the process which runs in separate process group, *
+ * because normally kill() wouldn't work                *
+ * Returns: status of the kill                          *
+ ********************************************************/
+gint  anjuta_util_kill(pid_t process_id, const gchar* signal_name);
+
+/********************************************************
+ * This function parses a string and returns a GList of *
+ * program args, just like shell parses the program args*
+ ********************************************************/
+GList* anjuta_util_parse_args_from_string (const gchar* string);
+
+/* Gets the relative filename w.r.t the given directory */
+gchar *get_relative_file_name(const gchar *dir, const gchar *file);
+
+/* Checks if the file exists in the given directory or any of it's subdirectories */
+gboolean is_file_in_dir(const gchar *file, const gchar *dir);
+
+GList *glist_path_dedup(GList *list);
+
+/* Check which gnome-terminal is installed
+ Returns: 0 -- No gnome-terminal
+ Returns: 1 -- Gnome1 gnome-terminal
+ Returns: 2 -- Gnome2 gnome-terminal */
+gint anjuta_util_check_gnome_terminal (void);
+
+/* String integer mapping utility functions */
+typedef struct _StringMap
+{
+	int type;
+	char *name;
+} StringMap;
+
+int type_from_string(StringMap *map, const char *str);
+const char *string_from_type(StringMap *map, int type);
+GList *glist_from_map(StringMap *map);
+
+void anjuta_util_color_from_string (const gchar * val,
+									guint8 * r, guint8 * g, guint8 * b);
+
+gchar *anjuta_util_string_from_color (guint8 r, guint8 g, guint8 b);
+
+/* Return must be freed. Works even if str is already UTF8 string */
+gchar *anjuta_util_convert_to_utf8 (const gchar *str);
+
+GtkWidget * anjuta_util_toolbar_append_button (GtkWidget *toolbar,
+											   const gchar *iconfile,
+											   const gchar *label,
+											   const gchar *tooltip,
+											   GtkSignalFunc callback,
+											   gpointer user_data);
+GtkWidget *
+anjuta_util_toolbar_append_stock (GtkWidget *toolbar, const gchar *stock_icon,
+								  const gchar *tooltip, GtkSignalFunc callback, 
+								  gpointer user_data);
+
+GtkWidget*
+anjuta_dialog_add_button (GtkDialog *dialog, const gchar* text,
+						  const gchar* stock_id, gint response_id);
+GtkWidget* 
+anjuta_button_new_with_stock_image (const gchar* text, const gchar* stock_id);
+
+/* Equivalent to gnome_execute_shell(), but does not overwrite
+ * SIGCHLD handler */
+pid_t anjuta_execute_shell (const gchar *dir, const gchar *command);
+
 #endif
