@@ -26,44 +26,39 @@
 typedef struct _ExprWatchGui ExprWatchGui;
 typedef struct _ExprWatch ExprWatch;
 
+enum {
+	WATCH_VARIABLE_COLUMN,
+	WATCH_VALUE_COLUMN,
+	WATCH_N_COLUMNS
+};
+
+
 struct _ExprWatchGui
 {
-    GtkWidget*   window;
     GtkWidget*   clist;
     GtkWidget*   menu_add;
     GtkWidget*   menu_remove;
     GtkWidget*   menu_clear;
     GtkWidget*   menu_update;
     GtkWidget*   menu_toggle;
+    GtkWidget*   menu_change;
     GtkWidget*   menu;
 };
 
 struct _ExprWatch
 {
   ExprWatchGui  widgets;
-  GList                    *exprs;
-  gint                      current_index;
-  gint                      count;
-  gboolean             is_showing;
-  gint                     win_pos_x, win_pos_y;
-  gint                     win_width, win_height;
 };
 
+struct watch_cb_data {
+	ExprWatch* ew;
+	GtkTreeIter* iter;
+};
+
+
 ExprWatch*
 expr_watch_new(void);
 
-GtkWidget*
-create_watch_menu(void);
-
-void
-create_expr_watch_gui(ExprWatch* ew);
-
-GtkWidget*
-create_watch_add_dialog(void);
-
-GtkWidget*
-create_eval_dialog(void);
-
 void
 expr_watch_clear(ExprWatch *ew);
 
@@ -79,20 +74,7 @@ expr_watch_update_controls(ExprWatch* ew);
 void
 expr_watch_destroy(ExprWatch*ew);
 
-gboolean
-expr_watch_save_yourself(ExprWatch* ew, FILE* stream);
-
-gboolean
-expr_watch_load_yourself(ExprWatch* ew, PropsID props);
-
-void
-expr_watch_show(ExprWatch * ew);
-
-void
-expr_watch_hide(ExprWatch * ew);
-
 void
 eval_output_arrived(GList *outputs, gpointer data);
 
 #endif
-
