@@ -1256,7 +1256,7 @@ static void printTotals (const clock_t *const timeStamps)
 	fputc ('\n', errout);
     }
 
-#ifdef DEBUG
+#ifdef TM_DEBUG
     fprintf (errout, "longest tag line = %lu\n",
 	    (unsigned long) TagFile.max.line);
 #endif
