@@ -279,7 +279,7 @@ gboolean tm_tags_custom_dedup(GPtrArray *tags_array, TMTagCompareFunc compare_fu
  \param name Name of the tag to locate.
  \param partial If TRUE, matches the first part of the name instead of doing exact match.
 */
-TMTag **tm_tags_find(GPtrArray *sorted_tags_array, const char *name, gboolean partial);
+TMTag **tm_tags_find(GPtrArray *sorted_tags_array, const char *name, gboolean partial, int * tagCount);
 
 /*!
  Completely frees an array of tags.
@@ -301,6 +301,18 @@ void tm_tag_destroy(TMTag *tag);
 */
 void tm_tag_free(gpointer tag);
 
+/*!
+ Returns the type of tag as a string
+ \param tag The tag whose type is required
+*/
+const char *tm_tag_type_name(const TMTag *tag);
+
+/*!
+ Returns the TMTagType given the name of the type. Reverse of tm_tag_type_name.
+ \param tag_name Name of the tag type
+*/
+TMTagType tm_tag_name_type(const char* tag_name);
+
 /*!
   Prints information about a tag to the given file pointer.
   \param tag The tag whose info is required.
