@@ -1,5 +1,4 @@
 /*
-*   $Id: options.c,v 1.1 2001/11/22 06:58:33 naba Exp $
 *
 *   Copyright (c) 1996-2001, Darren Hiebert
 *
@@ -145,7 +144,7 @@ optionValues Option = {
     FALSE,		/* --totals */
     FALSE,		/* --line-directives */
 	FALSE,
-#ifdef DEBUG
+#ifdef TM_DEBUG
     0, 0		/* -D, -b */
 #endif
 };
@@ -156,12 +155,12 @@ optionValues Option = {
 
 static optionDescription LongOptionDescription [] = {
  {1,"  -a   Append the tags to an existing tag file."},
-#ifdef DEBUG
+#ifdef TM_DEBUG
  {1,"  -b <line>"},
  {1,"       Set break line."},
 #endif
  {0,"  -B   Use backward searching patterns (?...?)."},
-#ifdef DEBUG
+#ifdef TM_DEBUG
  {1,"  -D <level>"},
  {1,"       Set debug level."},
 #endif
@@ -321,7 +320,7 @@ static const char *const Features [] = {
 #if (defined (MSDOS) || defined (WIN32) || defined (OS2)) && defined (UNIX_PATH_SEPARATOR)
     "unix-path-separator",
 #endif
-#ifdef DEBUG
+#ifdef TM_DEBUG
     "debug",
 #endif
     NULL
@@ -1016,7 +1015,7 @@ static void readIgnoreList (const char *const list)
     eFree (newList);
 }
 
-static void addIgnoreListFromFile (const char *const fileName)
+void addIgnoreListFromFile (const char *const fileName)
 {
     stringList* tokens = stringListNewFromFile (fileName);
     if (Option.ignore == NULL)
@@ -1400,7 +1399,7 @@ static void processShortOption (const char *const option,
 	    checkOptionOrder (option);
 	    Option.append = TRUE;
 	    break;
-#ifdef DEBUG
+#ifdef TM_DEBUG
 	case 'b':
 	    if (atol (parameter) < 0)
 		error (FATAL, "-%s: Invalid line number", option);
