@@ -97,7 +97,7 @@ typedef struct sOptionValues {
     boolean printTotals;    /* --totals  print cumulative statistics */
     boolean lineDirectives; /* --linedirectives  process #line directives */
 	boolean nestFunction; /* --nest Nest inside function blocks for tags */
-#ifdef DEBUG
+#ifdef TM_DEBUG
     long debugLevel;	    /* -D  debugging output */
     unsigned long breakLine;/* -b  source line at which to call lineBreak() */
 #endif
@@ -137,6 +137,8 @@ extern void readOptionConfiguration (void);
 extern void initOptions (void);
 extern void freeOptionResources (void);
 
+void addIgnoreListFromFile (const char *const fileName);
+
 #endif	/* _OPTIONS_H */
 
 /* vi:set tabstop=8 shiftwidth=4: */
