@@ -51,14 +51,14 @@ boolean isident(int c)
 	return (boolean) (((c >= 'A') && (c <= 'Z')) ||
 	  ((c >= 'a') && (c <= 'z')) ||
 	  ((c >= '0') && (c <= '9')) ||
-	  (c == '_'));
+	  (c == '_') || (c == '$'));
 }
 
 boolean isident1(int c)
 {
 	return (boolean) (((c >= 'A') && (c <= 'Z')) ||
 	  ((c >= 'a') && (c <= 'z')) ||
-	  (c == '_') || (c == '~'));
+	  (c == '_') || (c == '~') || (c == '$'));
 }
 
 static void setInputFileName (const char *const fileName)
