@@ -105,7 +105,7 @@ static int get_tag_type(const char *tag_name)
 		else if (cmp < 0)
 			break;
 	}
-#ifdef DEBUG
+#ifdef TM_DEBUG
 	fprintf(stderr, "Unknown tag type %s\n", tag_name);
 #endif
 	return tm_tag_undef_t;
@@ -161,7 +161,7 @@ gboolean tm_tag_init(TMTag *tag, TMSourceFile *file, const tagEntryInfo *tag_ent
 				tag->atts.entry.access = TAG_ACCESS_DEFAULT;
 			else
 			{
-#ifdef DEBUG
+#ifdef TM_DEBUG
 				g_warning("Unknown access type %s", tag_entry->extensionFields.access);
 #endif
 				tag->atts.entry.access = TAG_ACCESS_UNKNOWN;
@@ -174,7 +174,7 @@ gboolean tm_tag_init(TMTag *tag, TMSourceFile *file, const tagEntryInfo *tag_ent
 				tag->atts.entry.impl = TAG_IMPL_VIRTUAL;
 			else
 			{
-#ifdef DEBUG
+#ifdef TM_DEBUG
 				g_warning("Unknown implementation %s", tag_entry->extensionFields.implementation);
 #endif
 				tag->atts.entry.impl = TAG_IMPL_UNKNOWN;
@@ -283,7 +283,7 @@ gboolean tm_tag_init_from_file(TMTag *tag, TMSourceFile *file, FILE *fp)
 					tag->atts.entry.impl = *(start + 1);
 					break;
 				default:
-#ifdef DEBUG
+#ifdef TM_DEBUG
 					g_warning("Unknown attribute %s", start + 1);
 #endif
 					break;
@@ -515,7 +515,7 @@ GPtrArray *tm_tags_extract(GPtrArray *tags_array, guint tag_types)
 
 void tm_tags_array_free(GPtrArray *tags_array, gboolean free_all)
 {
-	if (NULL != tags_array)
+	if (tags_array)
 	{
 		guint i;
 		for (i = 0; i < tags_array->len; ++i)
@@ -527,10 +527,11 @@ void tm_tags_array_free(GPtrArray *tags_array, gboolean free_all)
 	}
 }
 
-TMTag **tm_tags_find(GPtrArray *sorted_tags_array, const char *name, gboolean partial)
+TMTag **tm_tags_find(GPtrArray *sorted_tags_array, const char *name, gboolean partial, int * tagCount)
 {
 	static TMTag *tag = NULL;
 	TMTag **result;
+	int tagMatches=0;
 	
 	if ((!sorted_tags_array) || (!sorted_tags_array->len))
 		return NULL;
@@ -544,16 +545,19 @@ TMTag **tm_tags_find(GPtrArray *sorted_tags_array, const char *name, gboolean pa
 	  , sizeof(gpointer), tm_tag_compare);
 	if (result)
 	{
-		for (; result >= (TMTag **) sorted_tags_array->pdata; -- result)
+		for (; result >= (TMTag **) sorted_tags_array->pdata; -- result) {
 			if (0 != tm_tag_compare(&tag, (TMTag **) result))
 				break;
+			++tagMatches;
+		}
+		*tagCount=tagMatches;
 		++ result;
 	}
 	s_partial = FALSE;
 	return (TMTag **) result;
 }
 
-static const char *tm_tag_type_name(TMTag *tag)
+const char *tm_tag_type_name(const TMTag *tag)
 {
 	g_return_val_if_fail(tag, NULL);
 	switch(tag->type)
@@ -582,6 +586,32 @@ static const char *tm_tag_type_name(TMTag *tag)
 	return NULL;
 }
 
+TMTagType tm_tag_name_type(const char* tag_name)
+{
+	g_return_val_if_fail(tag_name, tm_tag_undef_t);
+
+	if (strcmp(tag_name, "class") == 0) return tm_tag_class_t;
+	else if (strcmp(tag_name, "enum") == 0) return tm_tag_enum_t;
+	else if (strcmp(tag_name, "enumval") == 0) return tm_tag_enumerator_t;
+	else if (strcmp(tag_name, "field") == 0) return tm_tag_field_t;
+	else if (strcmp(tag_name, "function") == 0) return tm_tag_function_t;
+	else if (strcmp(tag_name, "interface") == 0) return tm_tag_interface_t;
+	else if (strcmp(tag_name, "member") == 0) return tm_tag_member_t;
+	else if (strcmp(tag_name, "method") == 0) return tm_tag_method_t;
+	else if (strcmp(tag_name, "namespace") == 0) return tm_tag_namespace_t;
+	else if (strcmp(tag_name, "package") == 0) return tm_tag_package_t;
+	else if (strcmp(tag_name, "prototype") == 0) return tm_tag_prototype_t;
+	else if (strcmp(tag_name, "struct") == 0) return tm_tag_struct_t;
+	else if (strcmp(tag_name, "typedef") == 0) return tm_tag_typedef_t;
+	else if (strcmp(tag_name, "union") == 0) return tm_tag_union_t;
+	else if (strcmp(tag_name, "variable") == 0) return tm_tag_variable_t;
+	else if (strcmp(tag_name, "extern") == 0) return tm_tag_externvar_t;
+	else if (strcmp(tag_name, "define") == 0) return tm_tag_macro_t;
+	else if (strcmp(tag_name, "macro") == 0) return tm_tag_macro_with_arg_t;
+	else if (strcmp(tag_name, "file") == 0) return tm_tag_file_t;
+	else return tm_tag_undef_t;
+}
+
 static const char *tm_tag_impl_name(TMTag *tag)
 {
 	g_return_val_if_fail(tag && (tm_tag_file_t != tag->type), NULL);
@@ -631,7 +661,7 @@ void tm_tag_print(TMTag *tag, FILE *fp)
 	if (tag->atts.entry.arglist)
 		fprintf(fp, "%s", tag->atts.entry.arglist);
 	if (tag->atts.entry.inheritance)
-		fprintf(fp, " : public %s", tag->atts.entry.inheritance);
+		fprintf(fp, " : from %s", tag->atts.entry.inheritance);
 	if ((tag->atts.entry.file) && (tag->atts.entry.line > 0))
 		fprintf(fp, "[%s:%ld]", tag->atts.entry.file->work_object.file_name
 		  , tag->atts.entry.line);
